/**
 * スクロールテーブルリスト構築クラス<br>
 * <p>
 * 通常のリスト形式のテーブルをスクロール可能なテーブルとして構築するインタフェースを提供します。<br>
 * </p>
 *
 * @author Kitagawa<br>
 *
 *<!--
 * 更新日		更新者				更新内容
 * 2012/07/02	Kitagawa			新規作成
 *-->
 */
/**
 * コンストラクタ<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>target : ターゲット要素又はセレクタ(ディフォルト:null)</li>
 * </ul>
 */
var ScrollTableList = function(options) {

	/** ユニークID */
	this.uid;

	/** 生成済みインスタンス */
	this.$created;

	/** ターゲット要素 */
	this.target;

	/** コンテナ幅 */
	this.width;

	/** コンテナ高さ */
	this.height;

	/** 固定行数 */
	this.fixRow;

	/** 固定列数 */
	this.fixCol;

	/** カラムサイズ定義 */
	this.columns;

	/**
	 * コンストラクタ処理
	 */
	var defaults = {
		target : void 0,
		width : void 0,
		height : void 0,
		fixRow : 0,
		fixCol : 0,
		columns : []
	};
	var mergedOptions = $.extend({}, defaults, options);
	{
		this.uid = "ScrollTableList." + UniqueID.generate();
		this.$created = void 0;
		this.target = mergedOptions.target;
		this.width = !mergedOptions.width ? void 0 : ElementUtil.getWidthPx(mergedOptions.width);
		this.height = !mergedOptions.height ? void 0 : ElementUtil.getHeightPx(mergedOptions.height);
		this.fixRow = mergedOptions.fixRow;
		this.fixCol = mergedOptions.fixCol;
		this.columns = mergedOptions.columns;
	}
};

/** イベントプレフィックス */
ScrollTableList.EVENT_PREFIX = "ScrollTableList.event";

/**
 * 指定されたオプションからスクロール可能なテーブルを構築します。<br>
 * @returns 正常に構築処理が完了した場合にtrueを返却
 */
ScrollTableList.prototype.create = function() {
	var undef = void 0;
	var id = this.__getUid();

	/*
	 * 最大行高さ取得
	 */
	var maxTrHeight = this.__getMaxRowHeight();

	/*
	 * 既に生成済みの場合は破棄
	 */
	this.destroy();

	/*
	 * コンテナ生成
	 */
	var $container = this.__createContainer(id, this.width, this.height);

	/*
	 * テーブルオブジェクト生成
	 */
	// 左上テーブルオブジェクト
	var $tableLT = this.__createEmptyTable(0, this.fixCol - 1);
	$tableLT.append(this.__createColgroup(0, this.fixCol - 1));
	$tableLT.append(this.__createTr(0, this.fixRow - 1, 0, this.fixCol - 1, maxTrHeight));
	// 右上テーブルオブジェクト
	var $tableRT = this.__createEmptyTable(this.fixCol, undef);
	$tableRT.append(this.__createColgroup(this.fixCol, undef));
	$tableRT.append(this.__createTr(0, this.fixRow - 1, this.fixCol, undef, maxTrHeight));
	// 左下テーブルオブジェクト
	var $tableLB = this.__createEmptyTable(0, this.fixCol - 1);
	$tableLB.append(this.__createColgroup(0, this.fixCol - 1));
	$tableLB.append(this.__createTr(this.fixRow, undef, 0, this.fixCol - 1, maxTrHeight));
	// 右下テーブルオブジェクト
	var $tableRB = this.__createEmptyTable(this.fixCol, undef);
	$tableRB.append(this.__createColgroup(this.fixCol, undef));
	$tableRB.append(this.__createTr(this.fixRow, undef, this.fixCol, undef, maxTrHeight));

	/*
	 * 各サイズ取得
	 */
	var ltWidth = ElementUtil.getWidthPx($tableLT) + 2;
	var ltHeight = ElementUtil.getHeightPx($tableLT) + 2;
	var rtWidth = ElementUtil.getWidthPx($tableRT) + 2;
	var rtHeight = ElementUtil.getHeightPx($tableRT) + 2;
	var lbWidth = ElementUtil.getWidthPx($tableLB) + 2;
	var lbHeight = ElementUtil.getHeightPx($tableLB) + 2;
	var rbWidth = ElementUtil.getWidthPx($tableRB) + 2;
	var rbHeight = ElementUtil.getHeightPx($tableRB) + 2;

	/*
	 * 左側パネル生成
	 */
	var $divL = this.__createFloatLeftContainer(id + "-left", undef, ltWidth, undef);
	var $divLTW = this.__createOverflowHiddenContainer(id + "-left-top-wrapper", ltWidth, undef);
	var $divLT = this.__createOverflowHiddenContainer(id + "-left-top", ltWidth, undef);
	$divLT.append($tableLT);
	$divLTW.append($divLT);
	$divL.append($divLTW);
	var $divLBW = this.__createOverflowHiddenContainer(id + "-left-bottom-wrapper", undef, this.height - ltHeight - ElementUtil.convertEmToPx(1));
	var $divLB = this.__createOverflowHiddenContainer(id + "-left-bottom", undef, this.height - ltHeight);
	$divLB.append($tableLB);
	$divLBW.append($divLB);
	$divL.append($divLBW);
	var divLWidth = ElementUtil.getWidthPx($divL);

	$container.append($divL);

	/*
	 * 右側パネル生成
	 */
	var $divR = this.__createContainer(id + "-right", undef, undef);
	var $divRTW = this.__createOverflowHiddenContainer(id + "-right-top-wrapper", this.width - ltWidth - ElementUtil.convertEmToPx(1), undef);
	var $divRT = this.__createOverflowHiddenContainer(id + "-right-top", this.width - ltWidth, undef);
	$divRT.append($tableRT);
	$divRTW.append($divRT);
	$divR.append($divRTW);
	var $divRBW = this.__createOverflowHiddenContainer(id + "-right-bottom-wrapper", this.width - ltWidth, this.height - ltHeight);
	var $divRB = this.__createOverflowScrollContainer(id + "-right-bottom", this.width - ltWidth, this.height - ltHeight);
	$divRB.append($tableRB);
	$divRBW.append($divRB);
	$divR.append($divRBW);
	$container.append($divR);

	/*
	 * スクロール連動イベント追加
	 */
	$divRB.unbind("scroll." + ScrollTableList.EVENT_PREFIX);
	$divRB.bind("scroll." + ScrollTableList.EVENT_PREFIX, function() {
		$divRT.scrollLeft($(this).scrollLeft());
		if ($divRT.scrollLeft() < $(this).scrollLeft()) {
			$divRT.css("position", "relative");
			$divRT.css("left", ($divRT.scrollLeft() - $(this).scrollLeft()) + "px");
		} else {
			$divRT.css("position", "static");
			$divRT.css("left", "0px");
		}

		$divLB.scrollTop($(this).scrollTop());
		if ($divLB.scrollTop() < $(this).scrollTop()) {
			$divLB.css("position", "relative");
			$divLB.css("top", ($divLB.scrollTop() - $(this).scrollTop()) + "px");
		} else {
			$divLB.css("position", "static");
			$divLB.css("top", "0px");
		}
	});

	/*
	 * コンテナ要素追加
	 */
	$container.insertAfter($(this.target));
	this.$created = $container;

	/*
	 * 元テーブル要素削除
	 */
	$(this.target).remove();

	return true;
};

/**
 * 生成済みのスクロールリストがある場合は破棄します。<br>
 * @returns 正常に終了した場合にtrueを返却
 */
ScrollTableList.prototype.destroy = function() {
	if (this.$created) {
		$(this.target).insertAfter(this.$created);
		this.$created.remove();
		this.$created = void 0;
	}
	return true;
};

/**
 * スクロールテーブル要素セットのIDを取得します。<br>
 * コンテナIDはターゲットとして指定されたテーブルオブジェクトがID属性を持っている場合はそれが設定されます。<br>
 * それ以外の場合はクラスインスタンスが生成された際に採番されたIDが返却されます。<br>
 * @returns スクロールテーブル要素セットのID
 */
ScrollTableList.prototype.__getUid = function() {
	var id = $(this.target).attr("id");
	if (!id || id === "") {
		id = this.uid;
	}
	return id;
};

/**
 * スクロールテーブル要素セットのIDを取得します。<br>
 * コンテナIDはターゲットとして指定されたテーブルオブジェクトがID属性を持っている場合はそれが設定されます。<br>
 * それ以外の場合はクラスインスタンスが生成された際に採番されたIDが返却されます。<br>
 * @returns スクロールテーブル要素セットのID
 */
ScrollTableList.prototype.__getMaxRowHeight = function() {
	// ターゲットテーブルクローン
	var $table = $(this.target).clone();

	var $dummy = $("<div/>");
	$dummy.css("display", "none");
	$dummy.append($table);
	$("body").append($dummy);

	var maxRowHeight = 0;
	var nowHeight = $dummy.outerHeight(true);
	$table.find("tr").each(function(indexTr, elementTr) {
		$(elementTr).remove();
		var diff = nowHeight - $dummy.outerHeight(true);
		nowHeight = $dummy.outerHeight(true);
		if (maxRowHeight < diff) {
			maxRowHeight = diff;
		}
	});

	return maxRowHeight;
};

/**
 * ターゲットテーブル要素から空のテーブル要素を生成します。<br>
 * @param colS 開始カラムインデックス
 * @param colE 終了カラムインデックス
 * @returns テーブルjQueryオブジェクト
 */
ScrollTableList.prototype.__createEmptyTable = function(colS, colE) {
	// パラメータ補正
	colS = typeof (colS) !== "undefined" ? colS : -1;
	colE = typeof (colE) !== "undefined" ? colE : this.columns ? this.columns.length - 1 : -1;

	// ピクセル幅取得
	var size = 0;
	$.each(this.columns, function(index, value) {
		if (!(index >= colS && index <= colE)) {
			return true;
		}
		var $sizeDiv = $("<div/>");
		$sizeDiv.css("display", "none");
		$sizeDiv.width(0);
		$sizeDiv.width(value);
		$("body").append($sizeDiv);
		size += $sizeDiv.width();
		$sizeDiv.remove();
	});

	// テーブル要素生成
	var $table = $(this.target).clone();
	$table.find("*").remove();
	$table.css("table-layout", "fixed");
	$table.width(size);

	return $table;
};

/**
 * カラムサイズオプションからカラムグループ要素を生成します。<br>
 * @param colS 開始カラムインデックス
 * @param colE 終了カラムインデックス
 * @returns カラムグループjQueryオブジェクト
 */
ScrollTableList.prototype.__createColgroup = function(colS, colE) {
	// パラメータ補正
	colS = typeof (colS) !== "undefined" ? colS : -1;
	colE = typeof (colE) !== "undefined" ? colE : this.columns ? this.columns.length - 1 : -1;

	// カラムグループ要素生成
	var $colgroup = $("<colgroup/>");
	$.each(this.columns, function(index, value) {
		if (!(index >= colS && index <= colE)) {
			return true;
		}
		var $col = $("<col/>");
		$col.css("width", value);
		$col.css("min-width", value);
		$col.css("max-width", value);
		$colgroup.append($col);
	});
	return $colgroup;
};

/**
 * ターゲットテーブル要素から指定範囲の行要素を生成します。<br>
 * @param rowS 開始行インデックス
 * @param rowE 終了行インデックス
 * @param colS 開始カラムインデックス
 * @param colE 終了カラムインデックス
 * @param maxTrHeight 最大行高さ
 * @returns 行jQueryオブジェクト配列
 */
ScrollTableList.prototype.__createTr = function(rowS, rowE, colS, colE, maxTrHeight) {
	// ターゲットテーブルクローン
	var $table = $(this.target).clone();

	// パラメータ補正
	colS = typeof (colS) !== "undefined" ? colS : -1;
	colE = typeof (colE) !== "undefined" ? colE : this.columns ? this.columns.length - 1 : -1;
	rowS = typeof (rowS) !== "undefined" ? rowS : -1;
	rowE = typeof (rowE) !== "undefined" ? rowE : $table.find("tr").length - 1;

	var $trs = [];
	var trsIndex = 0;
	$table.find("tr").each(function(indexTr, elementTr) {
		if (!(indexTr >= rowS && indexTr <= rowE)) {
			return true;
		}
		var $tr = $(elementTr).clone();
		$tr.find("*").remove();
		$(elementTr).find("td,th").each(function(indexTd, elementTd) {
			if (!(indexTd >= colS && indexTd <= colE)) {
				return true;
			}
			// 要素コンテナ生成
			var $tdDiv = $("<div/>");
			$tdDiv.css("margin", "0px");
			$tdDiv.css("padding", "0px");
			$tdDiv.append($(elementTd).clone().html());
			// カラム要素生成
			var $td = $(elementTd).clone();
			$td.html("");
			// 列行要素追加
			$td.append($tdDiv);
			$td.attr("nowrap", "nowrap");
			$td.attr("padding", "0px");
			$td.attr("margin", "0px");
			$tr.append($td);
			// 高さ設定
			if (maxTrHeight) {
				$tr.height(maxTrHeight);
			}
		});
		$trs[trsIndex++] = $tr;
	});
	return $trs;
};

/**
 * 汎用的なブロックコンテナパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns スクロールテーブルを保持するコンテナjQueryオブジェクト
 */
ScrollTableList.prototype.__createContainer = function(id, width, height) {
	// コンテナ要素生成
	var $div = $("<div/>");
	if (id && id !== "") {
		$div.attr("id", id);
	}
	if (width && width != "") {
		$div.width(width);
	}
	if (height && height != "") {
		$div.height(height);
	}
	$div.css("margin", 0);
	$div.css("padding", 0);
	$div.css("display", "block");
	return $div;
};

/**
 * オーバーフロー時に非表示とするパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createOverflowHiddenContainer = function(id, width, height) {
	var $div = this.__createContainer(id, width, height);
	$div.css("overflow", "hidden");
	return $div;
};

/**
 * オーバーフロー時にスクロールさせるとするパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createOverflowScrollContainer = function(id, width, height) {
	var $div = this.__createContainer(id, width, height);
	$div.css("overflow", "scroll");
	return $div;
};

/**
 * 左フローとさせるパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createFloatLeftContainer = function(id, width, height) {
	var $div = this.__createContainer(id, width, height);
	$div.css("float", "left");
	return $div;
};

/**
 * 右フローとさせるパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createFloatRightContainer = function(id, width, height) {
	var $div = this.__createContainer(id, width, height);
	$div.css("float", "right");
	return $div;
};
