/**
 * スクロールテーブルリスト構築クラス<br>
 * <p>
 * 通常のリスト形式のテーブルをスクロール可能なテーブルとして構築するインタフェースを提供します。<br>
 * </p>
 *
 * @author Kitagawa<br>
 *
 *<!--
 * 更新日		更新者				更新内容
 * 2012/07/02	Kitagawa			新規作成
 *-->
 */
/**
 * コンストラクタ<br>
 * <ul>
 * <li>target : ターゲット要素又はセレクタ(ディフォルト:null)</li>
 * <li>width : 生成するテーブルリストコンテナ幅(ディフォルト:null)</li>
 * <li>height : 生成するテーブルリストコンテナ高さ(ディフォルト:null)</li>
 * <li>fixRow : 非スクロールとする固定行数(ディフォルト:null)</li>
 * <li>fixCol : 非スクロールとする固定列数(ディフォルト:null)</li>
 * <li>columnWidths : 列幅配列(ディフォルト:null)</li>
 * <li>columnHeight : 行高さ(ディフォルト:1em)</li>
 * </ul>
 * @param {Object} options 動作オプション<br>
 */
var ScrollTableList = function(options) {

	/** ユニークID */
	this.uid;

	/** ターゲット要素 */
	this.target;

	/** コンテナ幅 */
	this.width;

	/** コンテナ高さ */
	this.height;

	/** 固定行数 */
	this.fixRow;

	/** 固定列数 */
	this.fixCol;

	/** カラムサイズ定義 */
	this.columnWidths;

	/** カラムサイズ定義 */
	this.columnHeight;

	/**
	 * コンストラクタ処理
	 */
	var defaults = {
		target : void 0,
		width : void 0,
		height : void 0,
		fixRow : 0,
		fixCol : 0,
		columnWidths : [],
		columnHeight : "1em"
	};
	var mergedOptions = $.extend({}, defaults, options);
	{
		this.uid = "ScrollTableList." + UniqueID.generate();
		this.target = mergedOptions.target;
		this.width = !mergedOptions.width ? void 0 : ElementUtil.getWidthPx(mergedOptions.width);
		this.height = !mergedOptions.height ? void 0 : ElementUtil.getHeightPx(mergedOptions.height);
		this.fixRow = mergedOptions.fixRow;
		this.fixCol = mergedOptions.fixCol;
		this.columnWidths = mergedOptions.columnWidths;
		this.columnHeight = mergedOptions.columnHeight;
	}
};

/** イベントプレフィックス */
ScrollTableList.EVENT_PREFIX = "ScrollTableList.event";

/** 生成済みフラグ */
ScrollTableList.CREATED_FLAG = "ScrollTableList.created";

/**
 * 指定されたオプションからスクロール可能なテーブルを構築します。<br>
 * @returns 正常に構築処理が完了した場合にtrueを返却
 */
ScrollTableList.prototype.create = function() {
	var undef = void 0;
	var scrollbarSize = 16;
	var id = $(this.target).attr("id") && $(this.target).attr("id") !== "" ? $(this.target).attr("id") : this.uid;
	var $target = $(this.target);
	//$target.css("border-spacing", "0px");
	//$target.css("border-collapse","collapse");

	/*
	 * 既に生成されたテーブルである場合はスキップ
	 */
	if ($target.attr(ScrollTableList.CREATED_FLAG) === "true") {
		return false;
	}

	/*
	 * テーブルセル実幅とオプションからセル幅情報を生成
	 */
	var columnWidths = this.__createColumnWidths($target, this.columnWidths);

	/*
	 * テーブルオブジェクトから加工用テーブルオブジェクトを生成
	 */
	var $table = this.__createCloneTable($target, this.width, columnWidths, this.columnHeight);
	var $tableLT = this.__createDividedTable($table, 0, 0, this.fixRow - 1, this.fixCol - 1);
	var $tableRT = this.__createDividedTable($table, 0, this.fixCol, this.fixRow - 1, columnWidths.length - 1);
	var $tableLB = this.__createDividedTable($table, this.fixRow, 0, void 0, this.fixCol - 1);
	var $tableRB = this.__createDividedTable($table, this.fixRow, this.fixCol, void 0, void 0);

	/*
	 * 分割テーブルコンテナオブジェクトを生成
	 */
	var leftWidth = $tableLT.width();
	if (Browser.getBrowserName() === "Chrome" || Browser.getBrowserName() === "Safari") {
		leftWidth += this.fixCol;
		leftWidth += 1;
	}
	var rightWidth = this.width - leftWidth;
	var $divLT = this.__createOverflowHiddenContainer(id + "-lt", leftWidth, void 0).append($tableLT);
	var $divRT = this.__createOverflowHiddenContainer(id + "-rt", rightWidth - scrollbarSize, void 0).append($tableRT);
	var $divLB = this.__createOverflowHiddenContainer(id + "-lb", leftWidth, this.height - scrollbarSize).append($tableLB);
	var $divRB = this.__createOverflowScrollContainer(id + "-rb", rightWidth, this.height).append($tableRB);

	/*
	 * スクロールオーバーフロー時の位置ずれ補正の為のコンテナ生成
	 */
	var $divWLT = this.__createOverflowHiddenContainer(id + "-wlt", leftWidth, void 0).append($divLT);
	var $divWRT = this.__createOverflowHiddenContainer(id + "-wrt", rightWidth, void 0).append($divRT);
	var $divWLB = this.__createOverflowHiddenContainer(id + "-wlb", leftWidth, this.height).append($divLB);
	var $divWRB = this.__createOverflowHiddenContainer(id + "-wrb", rightWidth, this.height).append($divRB);

	/*
	 * 配置コンテナオブジェクト生成
	 */
	//var $divL = this.__createFloatLeftContainer(id + "-l", $divWLT.width(), this.height).append($divWLT).append($divWLB);
	//var $divR = this.__createFloatRightContainer(id + "-r", $divWRT.width(), this.height).append($divWRT).append($divWRB);
	var $divL = this.__createInlineBlockContainer(id + "-l", $divWLT.width(), this.height).append($divWLT).append($divWLB);
	var $divR = this.__createInlineBlockContainer(id + "-r", $divWRT.width(), this.height).append($divWRT).append($divWRB);

	/*
	 * スクロールイベントバインド
	 */
	$divLB.unbind("mousewheel." + ScrollTableList.EVENT_PREFIX);
	$divLB.bind("mousewheel." + ScrollTableList.EVENT_PREFIX, function(ev, delta, deltaX, deltaY) {
		$divRB.scrollTop($divRB.scrollTop() - Math.round(delta * 20));
		$divRB.scroll();
	});
	$divRB.unbind("scroll." + ScrollTableList.EVENT_PREFIX);
	$divRB.bind("scroll." + ScrollTableList.EVENT_PREFIX, function() {
		$divRT.scrollLeft($(this).scrollLeft());
		if ($divRT.scrollLeft() < $(this).scrollLeft()) {
			$divRT.css("position", "relative");
			$divRT.css("left", ($divRT.scrollLeft() - $(this).scrollLeft()) + "px");
		} else {
			$divRT.css("position", "static");
			$divRT.css("left", "0px");
		}
		$divLB.scrollTop($(this).scrollTop());
		if ($divLB.scrollTop() < $(this).scrollTop()) {
			$divLB.css("position", "relative");
			$divLB.css("top", ($divLB.scrollTop() - $(this).scrollTop()) + "px");
		} else {
			$divLB.css("position", "static");
			$divLB.css("top", "0px");
		}
	});

	/*
	 * 全体コンテナオブジェクト生成
	 */
	var $container = this.__createBlockContainer(id, this.width, this.height).append($divL).append($divR);
	$container.attr(ScrollTableList.CREATED_FLAG, "true");

	/*
	 * 配置テーブルオブジェクト毎のセル編集
	 */
	$container.insertAfter($target);

	/*
	 * 元テーブルオブジェクト破棄
	 */
	$target.remove();

	return true;
};

/**
 * 汎用的なブロックコンテナパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns スクロールテーブルを保持するコンテナjQueryオブジェクト
 */
ScrollTableList.prototype.__createBlockContainer = function(id, width, height) {
	// コンテナ要素生成
	var $div = $("<div/>");
	$div.css("padding", "0px");
	$div.css("margin", "0px");
	$div.css("display", "block");
	$div.css("position", "static");
	if (id && id !== "") {
		$div.attr("id", id);
	}
	if (width && width != "") {
		$div.width(width);
	}
	if (height && height != "") {
		$div.height(height);
	}
	return $div;
};

/**
 * 汎用的なインラインブロックコンテナパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns スクロールテーブルを保持するコンテナjQueryオブジェクト
 */
ScrollTableList.prototype.__createInlineBlockContainer = function(id, width, height) {
	// コンテナ要素生成
	var $div = $("<div/>");
	$div.css("padding", "0px");
	$div.css("margin", "0px");
	$div.css("display", "inline-block");
	$div.css("position", "static");
	if (id && id !== "") {
		$div.attr("id", id);
	}
	if (width && width != "") {
		$div.width(width);
	}
	if (height && height != "") {
		$div.height(height);
	}
	return $div;
};

/**
 * カラム情報として設定するdivタグ要素を生成します。<br>
 * @param html 内部HTML文字列
 * @returns {Object} divタグ要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createCellContainer = function(html) {
	var $div = this.__createBlockContainer(void 0, void 0, void 0);
	$div.css("overflow", "hidden");
	$div.html(html);
	return $div;
};

/**
 * オーバーフロー時に非表示とするパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns {Object} パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createOverflowHiddenContainer = function(id, width, height) {
	var $div = this.__createBlockContainer(id, width, height);
	$div.css("overflow", "hidden");
	return $div;
};

/**
 * オーバーフロー時にスクロールさせるとするパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns {Object} パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createOverflowScrollContainer = function(id, width, height) {
	var $div = this.__createBlockContainer(id, width, height);
	$div.css("overflow", "scroll");
	return $div;
};

/**
 * 左フローとさせるパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns {Object} パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createFloatLeftContainer = function(id, width, height) {
	var $div = this.__createBlockContainer(id, width, height);
	$div.css("float", "left");
	return $div;
};

/**
 * 右フローとさせるパネル要素を生成します。<br>
 * @param id エレメントID
 * @param width 幅
 * @param height 高さ
 * @returns {Object} パネル要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createFloatRightContainer = function(id, width, height) {
	var $div = this.__createBlockContainer(id, width, height);
	$div.css("float", "right");
	return $div;
};

/**
 * ターゲットテーブル要素から処理用のクローンテーブル要素を生成します。<br>
 * @param baseTable テーブル要素オブジェクト
 * @param tableWidth テーブル幅
 * @param columnWidths カラム幅配列
 * @param columnHeihgt カラム高さ
 * @returns {Object} テーブルjQueryオブジェクト
 */
ScrollTableList.prototype.__createCloneTable = function(baseTable, tableWidth, columnWidths, columnHeihgt) {
	var _this_ = this;
	var $table = $(baseTable).clone();
	var $colgroup = _this_.__createColgroup(columnWidths);
	$table.removeAttr("id");
	$table.removeAttr("name");
	$table.find("colgroup").remove();
	$colgroup.insertBefore($table.children()[0]);
	$table.css("table-layout", "fixed");
	$table.css("margin", "0px");
	$table.width(tableWidth);
	$table.find("tr").each(function(i, e) {
		$(this).height(columnHeihgt);
		$(this).find("th,td").each(function(i, e) {
			$(this).height(columnHeihgt);
			$(this).css("overflow", "hidden");
			$(this).attr("nowrap", "nowrap");
			var $div = _this_.__createCellContainer($(this).html());
			$(this).html("");
			$(this).append($div);
		});
	});
	return $table;
};

/**
 * 指定されたテーブルからカラム幅の実サイズ配列を生成します。<br>
 * また、個別のカラム幅配列が指定されている場合はそれらのサイズ情報をマージして提供します。<br>
 * @param baseTable テーブル要素オブジェクト
 * @param optionWidths オプション指定カラム幅配列
 * @returns {Array} カラム幅の実サイズ配列
 */
ScrollTableList.prototype.__createColumnWidths = function(baseTable, optionWidths) {
	var columnWidths = [];
	$(baseTable).find("tr").each(function(ri, re) {
		if (ri > 0) {
			return false;
		}
		$(this).find("th,td").each(function(ci, ce) {
			columnWidths[ci] = $(this).width() + "px";
		});
	});
	if (optionWidths) {
		if (optionWidths.length > columnWidths.length) {
			for ( var i = 0; i <= columnWidths.length - 1; i++) {
				columnWidths[i] = optionWidths[i];
			}
		} else {
			for ( var i = 0; i <= optionWidths.length - 1; i++) {
				columnWidths[i] = optionWidths[i];
			}
		}
	}
	return columnWidths;
};

/**
 * 指定されたカラム幅配列からcolgroupタグ要素を生成します。<br>
 * @param columnWidths カラム幅配列
 * @returns {Object} colgroupタグ要素jQueryオブジェクト
 */
ScrollTableList.prototype.__createColgroup = function(columnWidths) {
	var $colgroup = $("<colgroup/>");
	if (columnWidths) {
		$(columnWidths).each(function(i, e) {
			var $col = $("<col/>");
			if (e === "*") {
				$col.attr("width", e);
			} else {
				$col.css("width", e);
				$col.css("min-width", e);
				$col.css("max-width", e);
			}
			$colgroup.append($col);
		});
	}
	return $colgroup;
};

/**
 * 指定されたテーブルから指定範囲セルのみのテーブルオブジェクトを生成します。<br>
 * @param baseTable テーブル要素オブジェクト
 * @param rowStart 範囲開始行
 * @param colStart 範囲開始列
 * @param rowEnd 範囲終了行
 * @param colEnd 範囲終了列
 * @returns {Object} テーブル要素オブジェクト
 */
ScrollTableList.prototype.__createDividedTable = function(baseTable, rowStart, colStart, rowEnd, colEnd) {
	var $table = $(baseTable).clone();
	$table.children().remove();

	rowStart = typeof (rowStart) === "undefined" ? 0 : rowStart;
	colStart = typeof (colStart) === "undefined" ? 0 : colStart;
	rowEnd = typeof (rowEnd) === "undefined" ? Number.MAX_VALUE : rowEnd;
	colEnd = typeof (colEnd) === "undefined" ? Number.MAX_VALUE : colEnd;

	// table幅設定
	var width = 0;
	var setWidth = true;
	$(baseTable).find("colgroup").find("col").each(function(i, e) {
		if (i >= colStart && i <= colEnd) {
			if ($(this).attr("width")) {
				setWidth = false;
			} else {
				width += ElementUtil.getWidthPx($(this).css("width"));
			}
		}
	});
	if (setWidth) {
		$table.width(width);
	}

	// colgroupタグ要素
	var $colgroup = $("<colgroup/>");
	$(baseTable).find("colgroup").find("col").each(function(i, e) {
		if (i >= colStart && i <= colEnd) {
			$colgroup.append($(this).clone());
		}
	});
	$table.append($colgroup);

	// tr,th,tdタグ要素
	$(baseTable).find("tr").each(function(ri, re) {
		if (ri >= rowStart && ri <= rowEnd) {
			//var $tr = $("<tr/>");
			var $tr = $(this).clone();
			$tr.children().remove();
			$(this).find("th,td").each(function(ci, ce) {
				if (ci >= colStart && ci <= colEnd) {
					$tr.append($(this).clone());
				}
			});
			$table.append($tr);
		}
	});

	return $table;
};
