/**
 * サブミット操作クラス<br>
 * <br>
 * ページ内のサブミット操作に関するインタフェースが提供されます。<br>
 *
 * @author Kitagawa<br>
 *
 *<!--
 * 更新日		更新者				更新内容
 * 2012/07/02	Kitagawa			新規作成
 *-->
 */

/**
 * コンストラクタ<br>
 */
var Submit = function() {
};

/** サブミット中フラグ */
Submit.submitted = 0;

/** サブミットクラスオプション */
Submit.option = {};

/** ステータスメッセージ更新タイマー */
Submit.refreshStatusMessageTimer = void 0;

/**
 * サブミット制御用の各種初期化設定を行います。<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>functionBeforSubmit : ファンクション(サブミット実行前処理)</li>
 * </ul>
 * @returns {Boolean} 正常終了した場合はtrueを返却
 */
Submit.initialize = function(options) {
	var defaults = {
		functionBeforSubmit : void 0,
		timeoutAjax : 10 * 60 * 1000
	// VPN越しで異常に時間がかかる場合もある為、長時間のタイムアウト指定としておく
	};
	var mergedOptions = $.extend({}, defaults, options);
	Submit.option = mergedOptions;
};

/**
 * ステータスメッセージを更新します。<br>
 * @returns {Boolean} 正常に処理が完了した場合にtrueを返却
 */
Submit.refreshStatusMessage = function() {
	try {
		if (Submit.submitted > 0) {
			window.status = "(残り" + this.submitted + "処理) 処理を実行しています";
		} else {
			window.status = "ページが表示されました";
			if (Submit.refreshStatusMessageTimer) {
				Submit.refreshStatusMessageTimer = void 0;
			}
		}
	} catch (e) {
		window.status = "";
		return false;
	}
	return true;
};

/**
 * ステータスメッセージ更新スレッドを開始します。<br>
 * @returns {Boolean} 正常に処理が完了した場合にtrueを返却
 */
Submit.startRefreshStatusMessageThread = function() {
	if (!Submit.refreshStatusMessageTimer) {
		Submit.refreshStatusMessageTimer = setInterval(function() {
			Submit.refreshStatusMessage();
		}, 500);
		Submit.refreshStatusMessage();
	}
	return true;
};

/**
 * フォームサブミット処理を実施します。<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>method : サブミット方法(ディフォルト:"post")</li>
 * <li>form : サブミット対象フォームセレクタ/jQueryオブジェクト(ディフォルト:form[0])</li>
 * <li>url : サブミット対象URL(コンテンツパスからの相対パス)(ディフォルト:"/")</li>
 * <li>editedConfirm : 編集済み確認(ディフォルト:false)</li>
 * <li>forbidDoubleSubmit : ダブルサブミット禁止フラグ(ディフォルト:true)</li>
 * <li>lockContent : コンテンツロックフラグ(ディフォルト:true)</li>
 * <li>ajax : Ajaxを用いたサブミット処理フラグ(ディフォルト:true)</li>
 * </ul>
 * @returns {Boolean} 正常終了した場合はtrueを返却
 */
Submit.doSubmit = function(options) {
	var defaults = {
		method : "post",
		form : $("form:first"),
		url : "/",
		editedConfirm : false,
		forbidDoubleSubmit : true,
		lockContent : true,
		ajax : true
	};
	var mergedOptions = $.extend({}, defaults, options);

	/*
	 * Ajaxを用いたサブミット処理フラグが設定されている場合はdoSubmitAjaxに処理を委譲
	 */
	if (mergedOptions.ajax) {
		return Submit.doSubmitAjax(mergedOptions);
	}

	var uid = UniqueID.generate();
	var lockLayer = new LockLayer({
		excludeEvents : [ "submit" ]
	});
	var $disabled = void 0;

	try {
		// ダブルサブミットチェック
		if (mergedOptions.forbidDoubleSubmit && Submit.submitted > 0) {
			alert(Message.get("FCRE00036"));
			return false;
		}

		// 編集済み確認
		if (Environment.FIELD_KEY_FIELD_EDITED !== "") {
			var editedFlag = $("#" + Environment.FIELD_KEY_FIELD_EDITED).val();
			if (mergedOptions.editedConfirm && editedFlag !== "") {
				if (!window.confirm(Message.get("FCRE00037"))) {
					return false;
				}
			}
		}

		// ダブルサブミット制御カウンタインクリメント
		Submit.submitted++;
		Submit.startRefreshStatusMessageThread();

		// 非活性項目の活性化(非活性フィールド強制サブミット対応)
		$disabled = $("input:disabled, textarea:disabled, select:disabled").removeAttr("disabled");

		// コンテンツロック
		if (mergedOptions.lockContent) {
			setTimeout(function() {
				lockLayer.lock();
			}, 0);
		}

		// サブミット実行前処理
		if (Submit.option.functionBeforSubmit) {
			Submit.option.functionBeforSubmit();
		}

		// サブミット処理
		var $targetForm = $(mergedOptions.form);
		$targetForm.attr("action", Environment.REQUEST_CONTEXT_PATH + Environment.SERVLET_PATH + mergedOptions.url);
		$targetForm.attr("method", mergedOptions.method);
		$targetForm.submit();
	} catch (e) {
		// コンテンツアンロック
		setTimeout(function() {
			lockLayer.unlock();
		}, 0);
		throw e;
	} finally {
		if ($disabled) {
			// 活性化した非活性フィールドを元に戻す
			$disabled.attr("disabled", "disabled");
		}
		// ダブルサブミット制御カウンタデクリメント
		Submit.submitted--;
	}

	return true;
};

/**
 * フォームサブミット処理をAjaxで実施します。<br>
 * 当メソッドでのフォームサブミットによる画面遷移はHTMLの動的変更によるもので画面自体の再描画は行いません。<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>method : サブミット方法(ディフォルト:"post")</li>
 * <li>form : サブミット対象フォームセレクタ/jQueryオブジェクト(ディフォルト:form[0])</li>
 * <li>url : サブミット対象URL(コンテンツパスからの相対パス)(ディフォルト:"/")</li>
 * <li>editedConfirm : 編集済み確認(ディフォルト:false)</li>
 * <li>forbidDoubleSubmit : ダブルサブミット禁止フラグ(ディフォルト:true)</li>
 * <li>lockContent : コンテンツロックフラグ(ディフォルト:true)</li>
 * </ul>
 * @returns {Boolean} 正常終了した場合はtrueを返却
 */
Submit.doSubmitAjax = function(options) {
	var defaults = {
		method : "post",
		form : $("form:first"),
		url : "/",
		editedConfirm : false,
		forbidDoubleSubmit : true,
		lockContent : true
	};
	var mergedOptions = $.extend({}, defaults, options);
	var uid = UniqueID.generate();
	var lockLayer = new LockLayer({
		excludeEvents : [ "submit" ]
	});
	var $disabled = void 0;

	try {
		// ダブルサブミットチェック
		if (mergedOptions.forbidDoubleSubmit && Submit.submitted > 0) {
			alert(Message.get("FCRE00036"));
			return false;
		}

		// 編集済み確認
		if (Environment.FIELD_KEY_FIELD_EDITED !== "") {
			var editedFlag = $("#" + Environment.FIELD_KEY_FIELD_EDITED).val();
			if (mergedOptions.editedConfirm && editedFlag !== "") {
				if (!window.confirm(Message.get("FCRE00037"))) {
					return false;
				}
			}
		}

		// ダブルサブミット制御カウンタインクリメント
		Submit.submitted++;
		Submit.startRefreshStatusMessageThread();

		// 非活性項目の活性化(非活性フィールド強制サブミット対応)
		$disabled = $("input:disabled, textarea:disabled, select:disabled").removeAttr("disabled");

		// コンテンツロック
		if (mergedOptions.lockContent) {
			setTimeout(function() {
				lockLayer.lock();
			}, 0);
		}

		// サブミット実行前処理
		if (Submit.option.functionBeforSubmit) {
			Submit.option.functionBeforSubmit();
		}

		// フォーム情報のシリアライズ
		var data = $(mergedOptions.form).serialize();

		// 活性化した非活性フィールドを元に戻す
		$disabled.attr("disabled", "disabled");

		// 配列プロパティは除去
		data = data.replace(/%5B/g, "[");
		data = data.replace(/%5D/g, "]");

		$.ajax({
			type : mergedOptions.method,
			timeout : mergedOptions.timeoutAjax,
			url : Environment.REQUEST_CONTEXT_PATH + Environment.SERVLET_PATH + mergedOptions.url,
			async : true,
			cache : false,
			dataType : "html",
			data : data,
			beforeSend : function(XMLHttpRequest) {
				return true;
			},
			error : function(XMLHttpRequest, textStatus, errorThrown) {
				var statusCode = XMLHttpRequest.status;
				alert(Message.get("FCRE00069", [ DateUtil.getLocalYYYYsMMsDDaHHcSS(), statusCode ]));
				return false;
			},
			success : function(data, dataType) {
				$("#mainContent").html("");
				$("#mainContent").html($(data).find("#mainContent"));
				return true;
			},
			complete : function(XMLHttpRequest, textStatus) {
				setTimeout(function() {
					lockLayer.unlock();
				}, 0);

				// ダブルサブミット制御カウンタデクリメント
				Submit.submitted--;

				return true;
			}
		});
	} catch (e) {
		// コンテンツアンロック
		setTimeout(function() {
			lockLayer.unlock();
		}, 0);
		throw e;
	} finally {
		if ($disabled) {
			// 活性化した非活性フィールドを元に戻す
			$disabled.attr("disabled", "disabled");
		}
	}

	return true;
};

/**
 * JSONを利用したフォームサブミット処理を実施します。<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>method : サブミット方法(ディフォルト:"post")</li>
 * <li>form : サブミット対象フォームセレクタ/jQueryオブジェクト(ディフォルト:form[0])</li>
 * <li>url : サブミット対象URL(コンテンツパスからの相対パス)(ディフォルト:"/")</li>
 * <li>editedConfirm : 編集済み確認(ディフォルト:false)</li>
 * <li>forbidDoubleSubmit : ダブルサブミット禁止フラグ(ディフォルト:false)</li>
 * <li>lockTarget : ロック対象エレメントセレクタ(ディフォルト:null)</li>
 * <li>beforeFunction : サブミット前実行関数(ディフォルト:null)</li>
 * <li>successFunction : サブミット処理成功後実行関数(ディフォルト:null)</li>
 * <li>completeFunction : サブミット後実行関数(ディフォルト:null)</li>
 * </ul>
 * @returns {Boolean} 正常終了した場合はtrueを返却
 */
Submit.doJSON = function(options) {
	var defaults = {
		method : "post",
		form : $("form:first"),
		url : "/",
		editedConfirm : false,
		forbidDoubleSubmit : false,
		lockTarget : null,
		beforeFunction : null,
		successFunction : null,
		completeFunction : null
	};
	var mergedOptions = $.extend({}, defaults, options);
	var uid = UniqueID.generate();
	var lockLayers = [];
	for ( var i = 0; i <= $(mergedOptions.lockTarget).length - 1; i++) {
		var lockLayer = new LockLayer({
			target : $(mergedOptions.lockTarget)[i],
			excludeEvents : [ "submit" ]
		});
		lockLayers[i] = lockLayer;
	}

	var $disabled = void 0;

	try {
		// ダブルサブミットチェック
		if (mergedOptions.forbidDoubleSubmit && Submit.submitted > 0) {
			alert(Message.get("FCRE00036"));
			return false;
		}

		// 編集済み確認
		if (Environment.FIELD_KEY_FIELD_EDITED !== "") {
			var editedFlag = $("#" + Environment.FIELD_KEY_FIELD_EDITED).val();
			if (mergedOptions.editedConfirm && editedFlag !== "") {
				if (!window.confirm(Message.get("FCRE00037"))) {
					return false;
				}
			}
		}

		// ダブルサブミット制御カウンタインクリメント
		Submit.submitted++;
		Submit.startRefreshStatusMessageThread();

		// 非活性項目の活性化(非活性フィールド強制サブミット対応)
		$disabled = $("input:disabled, textarea:disabled, select:disabled").removeAttr("disabled");

		// フォーム情報のシリアライズ
		var data = $(mergedOptions.form).serialize();

		// 活性化した非活性フィールドを元に戻す
		$disabled.attr("disabled", "disabled");

		// 配列プロパティは除去
		data = data.replace(/%5B/g, "[");
		data = data.replace(/%5D/g, "]");

		// コンテンツロック
		for ( var i = 0; i <= lockLayers.length - 1; i++) {
			var lockLayer = lockLayers[i];
			setTimeout(function() {
				lockLayer.lock();
			}, 0);
		}

		// サブミット実行前処理
		if (Submit.option.functionBeforSubmit) {
			Submit.option.functionBeforSubmit();
		}

		// サブミット処理
		$.ajax({
			type : mergedOptions.method,
			timeout : mergedOptions.timeoutAjax,
			url : Environment.REQUEST_CONTEXT_PATH + Environment.SERVLET_PATH + mergedOptions.url,
			async : true,
			cache : false,
			dataType : "json",
			data : data,
			beforeSend : function(XMLHttpRequest) {
				if (mergedOptions.beforeFunction) {
					mergedOptions.beforeFunction(XMLHttpRequest);
				}
				return true;
			},
			error : function(XMLHttpRequest, textStatus, errorThrown) {
				/*
				 * InternetExplorer(WinInet)固有のエラーレスポンスの一部を無視してリトライさせる
				 * (http://support.microsoft.com/kb/193625参照)
				 */
				var statusCode = XMLHttpRequest.status;
				if (false //
						|| statusCode === 12030 // ERROR_INTERNET_CONNECTION_ABORTED
						|| statusCode === 12152 // ERROR_HTTP_INVALID_SERVER_RESPONSE
				) {
					Submit.doJSON(mergedOptions);
					return true;
				}
				if (window.event) {
					if (window.event.type === "readystatechange") {
						alert(Message.get("FCRE00066"));
					}
				}
				if (textStatus === "parsererror") {
					alert(Message.get("FCRE00067", [ errorThrown ]));
				}
				if (textStatus === "timeout") {
					alert(Message.get("FCRE00068"));
				}
				alert(Message.get("FCRE00069", [ DateUtil.getLocalYYYYsMMsDDaHHcSS(), statusCode ]));

				return false;
			},
			success : function(data, dataType) {
				var errorMsg = "";
				for ( var i = 0; i <= data.errorActionMessages.length - 1; i++) {
					errorMsg += data.errorActionMessages[i];
					errorMsg += "\n";
				}
				if (errorMsg != "") {
					alert(errorMsg);
				}

				var informationMsg = "";
				for ( var i = 0; i <= data.informationActionMessages.length - 1; i++) {
					informationMsg += data.informationActionMessages[i];
					informationMsg += "\n";
				}
				if (informationMsg != "") {
					alert(informationMsg);
				}

				var warningMsg = "";
				for ( var i = 0; i <= data.warningActionMessages.length - 1; i++) {
					warningMsg += data.warningActionMessages[i];
					warningMsg += "\n";
				}
				if (warningMsg != "") {
					alert(warningMsg);
				}

				var noticeMsg = "";
				for ( var i = 0; i <= data.noticeActionMessages.length - 1; i++) {
					noticeMsg += data.noticeActionMessages[i];
					noticeMsg += "\n";
				}
				if (noticeMsg != "") {
					alert(noticeMsg);
				}

				var debugMsg = "";
				for ( var i = 0; i <= data.debugActionMessages.length - 1; i++) {
					debugMsg += data.debugActionMessages[i];
					debugMsg += "\n";
				}
				if (debugMsg != "") {
					alert(debugMsg);
				}

				var traceMsg = "";
				for ( var i = 0; i <= data.traceActionMessages.length - 1; i++) {
					traceMsg += data.traceActionMessages[i];
					traceMsg += "\n";
				}
				if (traceMsg != "") {
					alert(traceMsg);
				}

				/*
				var hiddenMsg = "";
				for ( var i = 0; i <= data.hiddenActionMessages.length - 1; i++) {
					hiddenMsg += data.hiddenActionMessages[i];
					hiddenMsg += "\n";
				}
				if (hiddenMsg != "") {
					alert(hiddenMsg);
				}
				*/

				if (mergedOptions.suncessFunction) {
					mergedOptions.suncessFunction(data.result, dataType);
				}

				return true;
			},
			complete : function(XMLHttpRequest, textStatus) {
				if (mergedOptions.completeFunction) {
					mergedOptions.completeFunction(XMLHttpRequest, textStatus);
				}

				// コンテンツアンロック
				for ( var i = 0; i <= lockLayers.length - 1; i++) {
					var lockLayer = lockLayers[i];
					setTimeout(function() {
						lockLayer.unlock();
					}, 0);
				}

				// ダブルサブミット制御カウンタデクリメント
				Submit.submitted--;

				return true;
			}
		});
	} catch (e) {
		// コンテンツアンロック
		for ( var i = 0; i <= lockLayers.length - 1; i++) {
			var lockLayer = lockLayers[i];
			setTimeout(function() {
				lockLayer.unlock();
			}, 0);
		}
		throw e;
	} finally {
		if ($disabled) {
			// 活性化した非活性フィールドを元に戻す
			$disabled.attr("disabled", "disabled");
		}
	}

	return true;
};

/**
 * ポップアップウィンドウを開いてフォームサブミット処理を実施します。<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>method : サブミット方法(ディフォルト:"post")</li>
 * <li>form : サブミット対象フォームセレクタ/jQueryオブジェクト(ディフォルト:form[0])</li>
 * <li>url : サブミット対象URL(コンテンツパスからの相対パス)(ディフォルト:"/")</li>
 * <li>windowName : ポップアップウィンドウ名(ディフォルト:未指定)</li>
 * <li>functionClose : ウィンドウクローズ時処理ファンクション(ディフォルト:なし)</li>
 * <li>editedConfirm : 編集済み確認(ディフォルト:false)</li>
 * <li>forbidDoubleSubmit : ダブルサブミット禁止フラグ(ディフォルト:false)</li>
 * <li>lockContent : コンテンツロックフラグ(ディフォルト:true)</li>
 * </ul>
 * @returns {Boolean} 正常終了した場合はtrueを返却
 */
Submit.doPopup = function(options) {
	var defaults = {
		method : "post",
		form : $("form:first"),
		url : "/",
		windowName : void 0,
		functionClose : void 0,
		editedConfirm : false,
		forbidDoubleSubmit : false,
		lockContent : false
	};
	var mergedOptions = $.extend({}, defaults, options);
	var uid = UniqueID.generate();
	var lockLayer = new LockLayer({
		excludeEvents : [ "submit" ]
	});
	var $disabled = void 0;

	try {
		// ダブルサブミットチェック
		if (mergedOptions.forbidDoubleSubmit && Submit.submitted > 0) {
			alert(Message.get("FCRE00036"));
			return false;
		}

		// 編集済み確認
		if (Environment.FIELD_KEY_FIELD_EDITED !== "") {
			var editedFlag = $("#" + Environment.FIELD_KEY_FIELD_EDITED).val();
			if (mergedOptions.editedConfirm && editedFlag !== "") {
				if (!window.confirm(Message.get("FCRE00037"))) {
					return false;
				}
			}
		}

		// ダブルサブミット制御カウンタインクリメント
		//Submit.submitted++;
		//Submit.startRefreshStatusMessageThread();

		// 非活性項目の活性化(非活性フィールド強制サブミット対応)
		$disabled = $("input:disabled, textarea:disabled, select:disabled").removeAttr("disabled");

		// コンテンツロック
		if (mergedOptions.lockContent) {
			setTimeout(function() {
				lockLayer.lock();
			}, 0);
		}

		// サブミット実行前処理
		if (Submit.option.functionBeforSubmit) {
			Submit.option.functionBeforSubmit();
		}

		// キャッシュ無効化ユニークキー生成
		var uniqueKey = UniqueID.generate();

		// ウィンドウID取得
		var windowId = $("#" + Environment.FIELD_KEY_WINDOW_ID).val();

		// 自ウィンドウトークン取得
		var token = "";
		if ($("#" + Environment.FIELD_KEY_TOKEN_ID) && $("#" + Environment.FIELD_KEY_TOKEN_ID).val() !== "undefined") {
			token = $("#" + Environment.FIELD_KEY_TOKEN_ID).val();
		}

		// アクションパス生成
		var actionPath = Environment.REQUEST_CONTEXT_PATH + Environment.SERVLET_PATH + mergedOptions.url;
		if (actionPath.indexOf("?") >= 0) {
			actionPath += "&";
			actionPath += Environment.FIELD_KEY_WINDOW_ID + "=";
			actionPath += windowId;
			actionPath += "&";
			actionPath += Environment.FIELD_KEY_POPUP_WINDOW + "=";
			actionPath += "1";
			actionPath += "&";
			actionPath += Environment.FIELD_KEY_TOKEN_ID + "=";
			actionPath += token;
		} else {
			actionPath += "?";
			actionPath += Environment.FIELD_KEY_WINDOW_ID + "=";
			actionPath += windowId;
			actionPath += "&";
			actionPath += Environment.FIELD_KEY_POPUP_WINDOW + "=";
			actionPath += "1";
			actionPath += "&";
			actionPath += Environment.FIELD_KEY_TOKEN_ID + "=";
			actionPath += token;
		}

		// URL構築
		var url = Environment.REQUEST_CONTEXT_PATH + Environment.JSP_POPUP_CONTAINER;
		url += "?";
		url += "_u=";
		url += uniqueKey;
		url += "&";
		url += Environment.REQUEST_KEY_ACTION_PATH;
		url += "=";
		url += escape(actionPath);
		url += "&";
		url += Environment.FIELD_KEY_POPUP_WINDOW;
		url += "=";
		url += "1";
		url += Environment.FIELD_KEY_TOKEN_ID + "=";
		url += token;

		// ポップアップウィンドウ表示
		var windowOption = "width=0, height=0, menubar=no, toolbar=no, scrollbars=no, location=no, status= no, resizable=yes";
		var w = void 0;
		if (Browser.getBrowserName() === "Chrome") {
			/*
			 * Google Chrome
			 * (IFrameを利用した親画面との値連携、ウィンドウサイズ設定が行えない為、ポップアップ自体のパスをリダイレクト)
			 */
			if (mergedOptions.windowName && mergedOptions.windowName !== "") {
				w = window.open(url, mergedOptions.windowName, windowOption);
			} else {
				w = window.open(url, new Date().getTime(), windowOption);
			}
		} else {
			/*
			 * Google Chrome以外
			 * (IFrameを利用した親画面との値連携、ウィンドウサイズ設定が行えるブラウザ)
			 */
			if (mergedOptions.windowName && mergedOptions.windowName !== "") {
				w = window.open(url, mergedOptions.windowName, windowOption);
			} else {
				w = window.open(url, new Date().getTime(), windowOption);
			}
		}

		/*
		 * 疑似的なウィンドウ表示時のコールバック処理
		 * (モーダルウィンドウコールバックと同様の処理を実現する)
		 */
		var callBackTimerId = void 0;
		callBackTimerId = setInterval(function() {
			if (w && w.closed) {
				// 疑似戻り値データ生成
				var virtualReturnValue = new Object();
				try {
					if (w.returnValue) {
						for ( var key in w.returnValue) {
							virtualReturnValue[key] = w.returnValue[key];
						}
					}
				} catch (e) {
					// window.returnValueへのアクセスが行えなかった場合の例外は無視
				}

				// 自ウィンドウトークン更新
				if ($("#" + Environment.FIELD_KEY_TOKEN_ID) && virtualReturnValue && virtualReturnValue[Environment.FIELD_KEY_TOKEN_ID] !== "undefined") {
					$("#" + Environment.FIELD_KEY_TOKEN_ID).val(virtualReturnValue[Environment.FIELD_KEY_TOKEN_ID]);
				}

				// トークン情報を戻り値データから削除
				if (virtualReturnValue) {
					delete virtualReturnValue[Environment.FIELD_KEY_TOKEN_ID];
					var count = 0;
					for ( var key in virtualReturnValue) {
						count++;
					}
					if (count == 0) {
						virtualReturnValue = void 0;
					}
				}

				// ポップアップクローズ時処理
				if (mergedOptions.functionClose) {
					mergedOptions.functionClose(virtualReturnValue);
				}

				// タイマークリア
				clearInterval(callBackTimerId);

				// コンテンツアンロック
				if (mergedOptions.lockContent) {
					setTimeout(function() {
						lockLayer.unlock();
					}, 0);
				}
			}
			return true;
		}, 0);
	} catch (e) {
		// コンテンツアンロック
		setTimeout(function() {
			lockLayer.unlock();
		}, 0);
		throw e;
	} finally {
		if ($disabled) {
			// 活性化した非活性フィールドを元に戻す
			$disabled.attr("disabled", "disabled");
		}
		// ダブルサブミット制御カウンタデクリメント
		//Submit.submitted--;
	}

	return true;
};

/**
 * ポップアップウィンドウを開いてフォームサブミット処理を実施します。<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>method : サブミット方法(ディフォルト:"post")</li>
 * <li>form : サブミット対象フォームセレクタ/jQueryオブジェクト(ディフォルト:form[0])</li>
 * <li>url : サブミット対象URL(コンテンツパスからの相対パス)(ディフォルト:"/")</li>
 * <li>functionClose : ウィンドウクローズ時処理ファンクション(ディフォルト:なし)</li>
 * <li>editedConfirm : 編集済み確認(ディフォルト:false)</li>
 * <li>forbidDoubleSubmit : ダブルサブミット禁止フラグ(ディフォルト:false)</li>
 * </ul>
 * @returns {Boolean} 正常終了した場合はtrueを返却
 */
Submit.doModal = function(options) {
	var defaults = {
		method : "post",
		form : $("form:first"),
		url : "/",
		functionClose : void 0,
		editedConfirm : false,
		forbidDoubleSubmit : false
	};
	var mergedOptions = $.extend({}, defaults, options);
	var uid = UniqueID.generate();
	var lockLayer = new LockLayer({
		excludeEvents : [ "submit" ]
	});
	var $disabled = void 0;

	try {
		if (Browser.getBrowserName() === "Chrome") {
			// Google Chromeの場合はshowModalDialogが正規対応していないためポップアップとする
			Submit.doPopup({
				method : mergedOptions.method,
				form : mergedOptions.form,
				url : mergedOptions.url,
				windowName : "",
				functionClose : mergedOptions.functionClose,
				editedConfirm : mergedOptions.editConfirm,
				forbidDoubleSubmit : mergedOptions.forbidDoubleSubmit,
				lockContent : true
			});
		} else {
			// ダブルサブミットチェック
			if (mergedOptions.forbidDoubleSubmit && Submit.submitted > 0) {
				alert(Message.get("FCRE00036"));
				return false;
			}

			// 編集済み確認
			if (Environment.FIELD_KEY_FIELD_EDITED !== "") {
				var editedFlag = $("#" + Environment.FIELD_KEY_FIELD_EDITED).val();
				if (mergedOptions.editedConfirm && editedFlag !== "") {
					if (!window.confirm(Message.get("FCRE00037"))) {
						return false;
					}
				}
			}

			// キャッシュ無効化ユニークキー生成
			var uniqueKey = UniqueID.generate();

			// ダブルサブミット制御カウンタインクリメント
			//Submit.submitted++;
			//Submit.startRefreshStatusMessageThread();

			// 非活性項目の活性化(非活性フィールド強制サブミット対応)
			$disabled = $("input:disabled, textarea:disabled, select:disabled").removeAttr("disabled");

			// サブミット実行前処理
			if (Submit.option.functionBeforSubmit) {
				Submit.option.functionBeforSubmit();
			}

			// コンテンツロック
			setTimeout(function() {
				lockLayer.lock();
			}, 0);

			// ウィンドウID取得
			var windowId = $("#" + Environment.FIELD_KEY_WINDOW_ID).val();

			// 自ウィンドウトークン取得
			var token = "";
			if ($("#" + Environment.FIELD_KEY_TOKEN_ID) && $("#" + Environment.FIELD_KEY_TOKEN_ID).val() !== "undefined") {
				token = $("#" + Environment.FIELD_KEY_TOKEN_ID).val();
			}

			// アクションパス生成
			var actionPath = Environment.REQUEST_CONTEXT_PATH + Environment.SERVLET_PATH + mergedOptions.url;
			if (actionPath.indexOf("?") >= 0) {
				actionPath += "&";
				actionPath += Environment.FIELD_KEY_WINDOW_ID + "=";
				actionPath += windowId;
				actionPath += "&";
				actionPath += Environment.FIELD_KEY_POPUP_WINDOW + "=";
				actionPath += "1";
				actionPath += "&";
				actionPath += Environment.FIELD_KEY_TOKEN_ID + "=";
				actionPath += token;
			} else {
				actionPath += "?";
				actionPath += Environment.FIELD_KEY_WINDOW_ID + "=";
				actionPath += windowId;
				actionPath += "&";
				actionPath += Environment.FIELD_KEY_POPUP_WINDOW + "=";
				actionPath += "1";
				actionPath += "&";
				actionPath += Environment.FIELD_KEY_TOKEN_ID + "=";
				actionPath += token;
			}

			// モーダルダイアログ引数生成
			var parameter = {
				url : actionPath,
				parentWindow : window
			};

			// URL構築
			var url = Environment.REQUEST_CONTEXT_PATH + Environment.JSP_POPUP_CONTAINER;
			url += "?";
			url += "_u=";
			url += uniqueKey;
			url += "&";
			url += Environment.FIELD_KEY_POPUP_WINDOW;
			url += "=";
			url += "1";
			url += Environment.FIELD_KEY_TOKEN_ID + "=";
			url += token;

			// モーダルダイアログ表示
			var windowOption = "dialogWidth:0px; dialogHeight:0px; scroll:no;";
			returnParams = showModalDialog(url, parameter, windowOption);

			// 自ウィンドウトークン更新
			if ($("#" + Environment.FIELD_KEY_TOKEN_ID) && returnParams && returnParams[Environment.FIELD_KEY_TOKEN_ID] !== "undefined") {
				$("#" + Environment.FIELD_KEY_TOKEN_ID).val(returnParams[Environment.FIELD_KEY_TOKEN_ID]);
			}

			// トークン情報を戻り値データから削除
			if (returnParams) {
				delete returnParams[Environment.FIELD_KEY_TOKEN_ID];
				var count = 0;
				for ( var key in returnParams) {
					count++;
				}
				if (count == 0) {
					returnParams = void 0;
				}
			}

			// ポップアップクローズ時処理
			if (mergedOptions.functionClose) {
				mergedOptions.functionClose(returnParams);
			}

			// コンテンツアンロック
			setTimeout(function() {
				lockLayer.unlock();
			}, 0);
		}
	} catch (e) {
		// コンテンツアンロック
		setTimeout(function() {
			lockLayer.unlock();
		}, 0);
		throw e;
	} finally {
		if ($disabled) {
			// 活性化した非活性フィールドを元に戻す
			$disabled.attr("disabled", "disabled");
		}
		// ダブルサブミット制御カウンタデクリメント
		//Submit.submitted--;
	}

	return true;
};

/**
 * ダウンロード処理を実施します。<br>
 * @param {Object} options 動作オプション<br>
 * <ul>
 * <li>method : サブミット方法(ディフォルト:"post")</li>
 * <li>form : サブミット対象フォームセレクタ/jQueryオブジェクト(ディフォルト:form[0])</li>
 * <li>url : サブミット対象URL(コンテンツパスからの相対パス)(ディフォルト:"/")</li>
 * <li>editedConfirm : 編集済み確認(ディフォルト:false)</li>
 * <li>forbidDoubleSubmit : ダブルサブミット禁止フラグ(ディフォルト:false)</li>
 * </ul>
 * @returns {Boolean} 正常終了した場合はtrueを返却
 */
Submit.doDownload = function(options) {
	var defaults = {
		method : "post",
		form : $("form:first"),
		url : "/",
		editedConfirm : false,
		forbidDoubleSubmit : false
	};
	var mergedOptions = $.extend({}, defaults, options);
	var uid = UniqueID.generate();
	var lockLayer = new LockLayer({
		excludeEvents : [ "submit" ]
	});
	var $disabled = void 0;

	try {
		// ダブルサブミットチェック
		if (mergedOptions.forbidDoubleSubmit && Submit.submitted > 0) {
			alert(Message.get("FCRE00036"));
			return false;
		}

		// 編集済み確認
		if (Environment.FIELD_KEY_FIELD_EDITED !== "") {
			var editedFlag = $("#" + Environment.FIELD_KEY_FIELD_EDITED).val();
			if (mergedOptions.editedConfirm && editedFlag !== "") {
				if (!window.confirm(Message.get("FCRE00037"))) {
					return false;
				}
			}
		}

		// ダブルサブミット制御カウンタインクリメント
		//Submit.submitted++;
		//Submit.startRefreshStatusMessageThread();

		// 非活性項目の活性化(非活性フィールド強制サブミット対応)
		$disabled = $("input:disabled, textarea:disabled, select:disabled").removeAttr("disabled");

		// サブミット実行前処理
		if (Submit.option.functionBeforSubmit) {
			Submit.option.functionBeforSubmit();
		}

		// サブミット処理
		var $targetForm = $(mergedOptions.form);
		$targetForm.attr("action", Environment.REQUEST_CONTEXT_PATH + Environment.SERVLET_PATH + mergedOptions.url);
		$targetForm.attr("method", mergedOptions.method);
		$targetForm.submit();
	} catch (e) {
		throw e;
	} finally {
		if ($disabled) {
			// 活性化した非活性フィールドを元に戻す
			$disabled.attr("disabled", "disabled");
		}
		// ダブルサブミット制御カウンタデクリメント
		//Submit.submitted--;
	}

	return true;
};
