package org.phosphoresce.webcore.core.config;

import java.io.Serializable;
import java.util.Properties;

import org.phosphoresce.lib.commons.util.PropertiesUtil;
import org.phosphoresce.lib.commons.util.ResourceUtil;
import org.phosphoresce.lib.commons.util.StringUtil;
import org.phosphoresce.webcore.core.GlobalConfigName;
import org.phosphoresce.webcore.core.GlobalConfiguration;
import org.phosphoresce.webcore.core.exception.WebcoreConfigurationException;

/**
 * ファイルリソースの拡張子に対応するMIMEタイプを提供するインタフェースを提供します。<br>
 * <p>
 * 当クラスは{@link Config}によってアプリケーション環境が初期化されていることを前提に、環境定義されているMIMEタイプ対応定義リソースを元に拡張子に対応するMIMEタイプを提供します。<br>
 * クラスの初期化は通常アプリケーション起動時に{@link GlobalConfiguration}によって行われ、アプリケーション起動後は永続的に情報が保持し続けられます。<br>
 * 例外的に環境設定リソースのリロードを行う場合は{@link MIMETypes#initialize()}によってリロードを行うことが可能ですが、{@link Config}インスタンス操作を排他したうえで、スレッドセーフに実施する必要があることに注意してください。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/06	Kitagawa		新規作成
 *-->
 */
public final class MIMETypes implements Serializable {

	/** 不明MIME代替タイプ */
	public static final String UNKOWN_MIME_TYPE = "application/octet-stream";

	/** クラスインスタンス */
	private static MIMETypes instance;

	/** プロパティオブジェクト */
	private Properties properties;

	/**
	 * コンストラクタ<br>
	 */
	private MIMETypes() {
		super();
	}

	/**
	 * 外部定義されたMIMEタイプ定義リソースからクラスインスタンスを初期化します。<br>
	 * このメソッドは{@link GlobalConfiguration}によって環境定義が初期化されていることが前提となります。<br>
	 */
	public static synchronized void initialize() {
		try {
			instance = new MIMETypes();
			instance.properties = new Properties();
			for (String value : Config.getStrings(GlobalConfigName.COMMON_RESROUCE_MIME_TYPE, true)) {
				if (!StringUtil.isEmpty(value)) {
					if (ResourceUtil.isExistResource(value)) {
						instance.properties.putAll(PropertiesUtil.load(value));
					}
				}
			}
		} catch (Throwable e) {
			throw new WebcoreConfigurationException("FCRE00024", new Object[] { Config.getString(GlobalConfigName.COMMON_RESROUCE_MIME_TYPE) }, e);
		}
	}

	/**
	 * クラスインスタンスを破棄します。<br>
	 */
	public static synchronized void destroy() {
		if (instance != null) {
			if (instance.properties != null) {
				instance.properties.clear();
			}
		}
		instance = null;
	}

	/**
	 * 指定された拡張子のMIMEタイプを取得します。<br>
	 * 未対応の拡張子の場合はapplication/octet-streamを返却します。<br>
	 * @param extension 拡張子
	 * @return MIMEタイプ文字列
	 */
	public static String getMIMEType(String extension) {
		if (instance == null) {
			return UNKOWN_MIME_TYPE;
		}
		String value = instance.properties.getProperty(extension);
		return StringUtil.isEmpty(value) ? UNKOWN_MIME_TYPE : value;
	}
}
