package org.phosphoresce.webcore.core.config;

import java.io.File;

import org.phosphoresce.lib.commons.util.FileUtil;
import org.phosphoresce.webcore.core.GlobalConfigName;
import org.phosphoresce.webcore.core.GlobalEnvironment;
import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.slf4j.LoggerFactory;

/**
 * サーバー上のリソースに対するアクセスインタフェースを提供します。<br>
 * <p>
 * アプリケーションサーバー上に存在するリソースに対するアクセスを行う際のインタフェースが提供されます。<br>
 * このクラスはアプリケーションからサーバー上のリソースに対して直接操作を行う際の処理を共通化するために設けられました。<br>
 * ユーザーはサーバー上のリソースを利用する際、このクラスを利用して操作することで、将来的にサーバーリソースアクセスの方式が変更になった際に、一貫したリファクタリングが可能となります。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2011/10/17	Kitagawa		新規作成
 *-->
 */
public final class ServerResource {

	/** ロガーオブジェクト */
	private static CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(ServerResource.class));

	/** テンポラリリソース基底パス */
	public static final String DOWNLOAD_PATH = Config.getString(GlobalConfigName.COMMON_GENERAL_DOWNLOAD_RESOURCE);

	/** テンポラリリソース基底パス */
	public static final String UPLOAD_PATH = Config.getString(GlobalConfigName.COMMON_GENERAL_UPLOAD_RESOURCE);

	/**
	 * コンストラクタ<br>
	 */
	private ServerResource() {
		super();
	}

	/**
	 * クラスを初期化します。<br>
	 */
	public static synchronized void initialize() {
		if (!new File(DOWNLOAD_PATH).exists()) {
			FileUtil.mkdir(DOWNLOAD_PATH);
			log.output("FCRE00059", DOWNLOAD_PATH);
		} else {
			log.output("FCRE00062", DOWNLOAD_PATH);
		}
		if (!new File(UPLOAD_PATH).exists()) {
			FileUtil.mkdir(UPLOAD_PATH);
			log.output("FCRE00060", UPLOAD_PATH);
		} else {
			log.output("FCRE00063", UPLOAD_PATH);
		}
	}

	/**
	 * 指定されたパスのサーバーリソースを取得します。<br>
	 * 指定されたパスが相対パスの場合、WEB-INFパス、アプリケーションコンテキストパスの順序でリソースを検索し、最初に見つかったリソースを返却します。<br>
	 * WEB-INFパス、コンテキストパスにリソースが存在しなかった場合は絶対パスとして指定パスを扱い、そのパスに存在するリソースを返却します。<br>
	 * @param path 取得対象のリソースパス
	 * @return サーバーリソースファイルオブジェクト
	 */
	public static File getFile(String path) {
		File file = null;
		file = new File(FileUtil.connectPath(GlobalEnvironment.getWebinfPath(), path));
		if (file.exists()) {
			return file;
		}
		file = new File(FileUtil.connectPath(GlobalEnvironment.getContentPath(), path));
		if (file.exists()) {
			return file;
		}
		file = new File(path);
		if (file.exists()) {
			return file;
		}
		return file;
	}

	/**
	 * 指定されたパスのサーバーリソースが存在するか判定します。<br>
	 * 指定されたパスが相対パスの場合、WEB-INFパス、アプリケーションコンテキストパスの順序でリソースを検索します。<br>
	 * WEB-INFパス、コンテキストパスにリソースが存在しなかった場合は絶対パスとして指定パスを扱います。<br>
	 * @param path 判定対象のリソースパス
	 * @return 指定されたパスのリソースが存在する場合にtrueを返却
	 */
	public static boolean existsFile(String path) {
		File file = getFile(path);
		return file != null && file.exists();
	}

	/**
	 * 指定されたパスの汎用ダウンロードパス配下に存在するリソースを取得します。<br>
	 * @param path 取得対象のリソースパス
	 * @return サーバーリソースファイルオブジェクト
	 */
	public static File getDownloadFile(String path) {
		return new File(FileUtil.connectPath(DOWNLOAD_PATH, path));
	}

	/**
	 * 指定されたパスの汎用ダウンロードパス配下にリソースが存在するか判定します。<br>
	 * @param path 判定対象のリソースパス
	 * @return 指定されたパスのリソースが存在する場合にtrueを返却
	 */
	public static boolean existsDownloadFile(String path) {
		File file = getDownloadFile(path);
		return file != null && file.exists();
	}

	/**
	 * 指定されたパスの汎用アップロードパス配下に存在するリソースを取得します。<br>
	 * @param path 取得対象のリソースパス
	 * @return サーバーリソースファイルオブジェクト
	 */
	public static File getUploadFile(String path) {
		return new File(FileUtil.connectPath(UPLOAD_PATH, path));
	}

	/**
	 * 指定されたパスの汎用アップロードパス配下にリソースが存在するか判定します。<br>
	 * @param path 判定対象のリソースパス
	 * @return 指定されたパスのリソースが存在する場合にtrueを返却
	 */
	public static boolean existsUploadFile(String path) {
		File file = getUploadFile(path);
		return file != null && file.exists();
	}
}
