package org.phosphoresce.webcore.core.logger;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import ch.qos.logback.classic.encoder.PatternLayoutEncoder;
import ch.qos.logback.classic.spi.ILoggingEvent;

/**
 * ログ出力時のパターンレイアウトでオフセットされた階層のトレース情報が利用されるようにエンコードする機能を提供します。<br>
 * <p>
 * コード指定によるログ出力のためにラップされたslf4jロガークラス({@link CodeConvertLogger})を利用した際に呼び出し階層を補正して、
 * 本来の階層のトレース情報が出力されるようにパターンレイアウトトレース階層をオフセットしてエンコードするクラスとして提供されます。<br>
 * </p>
 * <p>
 * オフセットされる階層は{@link CodeConvertLogger}のメソッドラップ階層に依存します。<br>
 * これはフレームワーククラス仕様が変更されない限り不変であるため、動的な環境変数では定義されず、当クラス内に隠蔽されています。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/08	Kitagawa		新規作成
 *-->
 */
public final class CallerOffsetPatternLayoutEncoder extends PatternLayoutEncoder {

	/** コールトレースオフセット */
	private static int OFFSET = 2;

	/**
	 * ログに対するエンコード処理を実行します。<br>
	 * @param event ロギングイベントオブジェクト
	 * @see ch.qos.logback.core.encoder.LayoutWrappingEncoder#doEncode(java.lang.Object)
	 */
	@Override
	public void doEncode(ILoggingEvent event) throws IOException {
		StackTraceElement[] elements = event.getCallerData();

		List<StackTraceElement> list = new LinkedList<StackTraceElement>();
		for (int i = OFFSET; i <= elements.length - 1; i++) {
			list.add(elements[i]);
		}

		for (int i = 0; i <= list.size() - 1; i++) {
			elements[i] = list.get(i);
		}

		super.doEncode(event);
	}
}
