package org.phosphoresce.webcore.core.logger;

import org.phosphoresce.webcore.core.config.StringResource;
import org.phosphoresce.webcore.core.config.StringResourceLevel;
import org.slf4j.Logger;

/**
 * コード定義されているメッセージを元にログ出力を行う機能を提供します。<br>
 * <p>
 * slf4jが提供するロガーオブジェクトをラップしたこのクラスは、通常のレベルごとのログ出力メソッドは提供せず、すべて{@link CodeConvertLogger#output(String)}をはじめとする共通の出力メソッドが提供されます。
 * この共通ログ出力メソッドに対して、指定する内容はすべてメッセージ定義されたメッセージコード(レベルサフィックスを除く)を指定します。<br>
 * ロガーは指定されたメッセージコードから自動的に出力レベルを判定し、レベルに該当するslf4jのログ出力メソッドを呼び出ます。<br>
 * 尚、出力するメッセージレベルは{@link StringResourceLevel}が提供する列挙型定数に対して以下の通りの出力レベルとなります。<br>
 * <table style="width: auto !important; border: 1px #000000 solid; !important;">
 * <tr>
 *   <td>レベル定数</td>
 *   <td>レベル定義</td>
 *   <td>コード定義</td>
 *   <td>slf4jログ</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#INFO}</td>
 *   <td>情報レベル</td>
 *   <td>*****-I</td>
 *   <td>info</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#ERROR}</td>
 *   <td>エラーレベル</td>
 *   <td>*****-E</td>
 *   <td>error</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#WARN}</td>
 *   <td>警告レベル</td>
 *   <td>*****-W</td>
 *   <td>warn</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#NOTICE}</td>
 *   <td>連絡レベル</td>
 *   <td>*****-N<li>
 *   <td>trace</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#DEBUG}</td>
 *   <td>デバッグレベル</td>
 *   <td>*****-D</td>
 *   <td>debug</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#TRACE}</td>
 *   <td>トレースレベル</td>
 *   <td>*****-T</td>
 *   <td>trace</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#HIDDEN}</td>
 *   <td>非出力ログレベル</td>
 *   <td>*****-H</td>
 *   <td>(ログ出力は行いません)</td>
 * </tr>
 * <tr>
 *   <td>{@link StringResourceLevel#NONE} / その他</td>
 *   <td>トレースレベル</td>
 *   <td>***** / -</td>
 *   <td>trace</td>
 * </tr>
 * </table>
 * <br>
 * </p>
 * <p>
 * <b>
 * 当クラスを利用したログ出力を行う場合、ラップされたロガーオブジェクトによる呼び出し階層を補正するため、logbackに対するエンコーダークラスに必ずCallerOffsetPatternLayoutEncoderを利用する必要があります。<br>
 * </b>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/08	Kitagawa		新規作成
 * 2013/07/24	Kitagawa		非出力とするログレベルを追加
 *-->
 */
public final class CodeConvertLogger {

	/** ロガーオブジェクト */
	private Logger logger;

	/**
	 * コンストラクタ<br>
	 * @param logger
	 */
	public CodeConvertLogger(Logger logger) {
		super();
		this.logger = logger;
	}

	/**
	 * メッセージログ出力処理を行います。<br>
	 * ログ出力時の呼び出し階層を一定とするために内部でメソッドの切り分けが行われています。<br>
	 * @param e 例外オブジェクト
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト配列
	 */
	private void outputCore(Throwable e, String code, Object... binds) {
		if (logger == null) {
			return;
		}
		StringResourceLevel level = StringResourceLevel.getDefinitionLevel(code);
		if (StringResourceLevel.INFO.equals(level)) {
			if (logger.isInfoEnabled()) {
				if (e != null) {
					logger.info(getMessage(code, binds), e);
				} else {
					logger.info(getMessage(code, binds));
				}
			}
		} else if (StringResourceLevel.ERROR.equals(level)) {
			if (logger.isErrorEnabled()) {
				if (e != null) {
					logger.error(getMessage(code, binds), e);
				} else {
					logger.error(getMessage(code, binds));
				}
			}
		} else if (StringResourceLevel.WARN.equals(level)) {
			if (logger.isWarnEnabled()) {
				if (e != null) {
					logger.warn(getMessage(code, binds), e);
				} else {
					logger.warn(getMessage(code, binds));
				}
			}
		} else if (StringResourceLevel.NOTICE.equals(level)) {
			if (logger.isTraceEnabled()) {
				if (e != null) {
					logger.trace(getMessage(code, binds), e);
				} else {
					logger.trace(getMessage(code, binds));
				}
			}
		} else if (StringResourceLevel.DEBUG.equals(level)) {
			if (logger.isDebugEnabled()) {
				if (e != null) {
					logger.debug(getMessage(code, binds), e);
				} else {
					logger.debug(getMessage(code, binds));
				}
			}
		} else if (StringResourceLevel.TRACE.equals(level)) {
			if (logger.isTraceEnabled()) {
				if (e != null) {
					logger.trace(getMessage(code, binds), e);
				} else {
					logger.trace(getMessage(code, binds));
				}
			}
		} else if (StringResourceLevel.HIDDEN.equals(level)) {
			/*
			 * {@link StringResourceLevel#HIDDEN}に合致するレベルはログ出力を行わない
			 */
		} else {
			if (logger.isTraceEnabled()) {
				if (e != null) {
					logger.trace(getMessage(code, binds), e);
				} else {
					logger.trace(getMessage(code, binds));
				}
			}
		}
	}

	/**
	 * 指定されたコードとバインドオブジェクトでメッセージを取得します。<br>
	 * ここで指定するメッセージコードはレベルサフィックス文字列を含まない純粋なメッセージコードを指定します。<br>
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト配列
	 * @return メッセージ文字列
	 */
	private String getMessage(String code, Object... binds) {
		if (code == null) {
			return "--------- | < Empty message code >";
		} else {
			return code + " | " + StringResource.getString(code, binds);
		}
	}

	/**
	 * 指定されたコードとバインドオブジェクトでメッセージログを出力します。<br>
	 * ここで指定するメッセージコードはレベルサフィックス文字列を含まない純粋なメッセージコードを指定します。<br>
	 * @param e 例外オブジェクト
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト配列
	 */
	public void output(Throwable e, String code, Object... binds) {
		outputCore(e, code, binds);
	}

	/**
	 * 指定されたコードとバインドオブジェクトでメッセージログを出力します。<br>
	 * ここで指定するメッセージコードはレベルサフィックス文字列を含まない純粋なメッセージコードを指定します。<br>
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト配列
	 */
	public void output(String code, Object... binds) {
		outputCore(null, code, binds);
	}

	/**
	 * 指定されたコードでメッセージログを出力します。<br>
	 * ここで指定するメッセージコードはレベルサフィックス文字列を含まない純粋なメッセージコードを指定します。<br>
	 * @param e 例外オブジェクト
	 * @param code メッセージコード
	 */
	public void output(Throwable e, String code) {
		outputCore(e, code, (Object[]) null);
	}

	/**
	 * 指定されたコードでメッセージログを出力します。<br>
	 * ここで指定するメッセージコードはレベルサフィックス文字列を含まない純粋なメッセージコードを指定します。<br>
	 * @param code メッセージコード
	 */
	public void output(String code) {
		outputCore(null, code, (Object[]) null);
	}
}
