package org.phosphoresce.webcore.core.servlet;

import java.net.InetAddress;
import java.text.DecimalFormat;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;

import org.phosphoresce.lib.commons.util.FileUtil;
import org.phosphoresce.webcore.core.GlobalConfigName;
import org.phosphoresce.webcore.core.GlobalConfiguration;
import org.phosphoresce.webcore.core.GlobalEnvironment;
import org.phosphoresce.webcore.core.config.Config;
import org.phosphoresce.webcore.core.config.StringResource;
import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.slf4j.LoggerFactory;

/**
 * フレームワークモジュールを利用したアプリケーションの起動を行う初期化処理を提供します。<br>
 * <p>
 * フレームワークを利用したアプリケーションを起動する場合、必ずこの起動サーブレットを利用する必要があります。<br>
 * 当サーブレットによるアプリケーション起動によって、環境定義リソースに定義された内容を基に{@link GlobalConfiguration}がアプリケーションの初期化処理を実行します。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/05	Kitagawa		新規作成
 *-->
 */
public class BootstrapServlet extends HttpServlet {

	/** ロガーオブジェクト */
	private CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(getClass()));

	/** パラメータキー(開発モード) */
	public static String PARAM_DEVELOP_MODE = "develop";

	/** パラメータキー(環境設定リソース) */
	public static String PARAM_CONFIG_FILE = "config";

	/**
	 * コンストラクタ<br>
	 */
	public BootstrapServlet() {
		super();
	}

	/**
	 * サーブレットの初期化処理を実行します。<br>
	 * @throws ServletException 初期化処理に失敗した場合にスローされます
	 * @see javax.servlet.GenericServlet#init()
	 */
	@Override
	public void init() throws ServletException {
		//super.init();
		double startTime = System.currentTimeMillis();
		try {
			// 環境変数オブジェクト初期化
			GlobalEnvironment.setDevelopMode(Boolean.parseBoolean(getInitParameter(PARAM_DEVELOP_MODE)));
			GlobalEnvironment.setContentPath(getServletContext().getRealPath("").replace("\\", "/"));
			GlobalEnvironment.setWebinfPath(FileUtil.connectPath(getServletContext().getRealPath(""), "WEB-INF").replace("\\", "/"));
			GlobalEnvironment.setContextPath(getServletContext().getContextPath());
			GlobalEnvironment.setServerHost(InetAddress.getLocalHost().getHostName());
			GlobalEnvironment.setConfigPath(GlobalEnvironment.bindEnv(getInitParameter(PARAM_CONFIG_FILE)));

			// 各種環境設定初期化
			GlobalConfiguration.configure();

			// ロガーオブジェクト再作成
			log = new CodeConvertLogger(LoggerFactory.getLogger(getClass()));

			// アプリケーション起動完了
			double endTime = System.currentTimeMillis();
			log.output("FCRE00008", GlobalEnvironment.bindEnv(getInitParameter(PARAM_CONFIG_FILE)));

			for (String name : Config.getStrings(GlobalConfigName.COMMON_RESOURCE_CONFIG_EXTENDS)) {
				log.output("FCRE00009", GlobalEnvironment.bindEnv(name));
			}
			log.output("FCRE00010", new DecimalFormat("0.000").format((endTime - startTime) / 1000D));

			if (GlobalEnvironment.isDevelopMode()) {
				log.output("FCRE00011");
			}

			// アプリケーション起動完了時間設定
			GlobalEnvironment.setStartedTime(System.currentTimeMillis());

			log.output("FCRE00007");
		} catch (Throwable e) {
			e.printStackTrace();
			throw new ServletException(StringResource.getString("FCRE00025"), e);
		}
	}

	/**
	 * サーブレットの破棄処理を実行します。<br>
	 * @see javax.servlet.Filter#destroy()
	 */
	@Override
	public void destroy() {
		GlobalConfiguration.destroy();
		log.output("FCRE00012");

		// ログ出力が必要となる為、メッセージリソース開放はBootstrapサーブレット破棄のタイミングとする
		StringResource.destroy();
	}
}
