package org.phosphoresce.webcore.ext.hibernate4;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import org.dom4j.Document;
import org.hibernate.cfg.Configuration;
import org.phosphoresce.lib.commons.util.DOM4JUtil;
import org.phosphoresce.lib.commons.util.PropertiesUtil;
import org.phosphoresce.lib.commons.util.StringUtil;
import org.phosphoresce.lib.jod.JODSOfficeProcessManager;
import org.phosphoresce.lib.poi.PoiWorkbookType;
import org.phosphoresce.lib.poi.adapter.PoiSheet;
import org.phosphoresce.lib.poi.adapter.PoiWorkbook;
import org.phosphoresce.webcore.core.GlobalEnvironment;
import org.phosphoresce.webcore.core.config.Config;
import org.phosphoresce.webcore.core.exception.WebcoreConfigurationException;
import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateConfigureException;
import org.phosphoresce.webcore.ext.hibernate4.model.SCode;
import org.phosphoresce.webcore.ext.hibernate4.model.SConfig;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.hibernate4.service.SystemMasterService;
import org.phosphoresce.webcore.ext.hibernate4.transaction.HibernateTransactionContext;
import org.phosphoresce.webcore.ext.hibernate4.transaction.HibernateTransactionManager;
import org.phosphoresce.webcore.ext.hibernate4.transaction.TransactionFactory;
import org.phosphoresce.webcore.ext.hibernate4.util.SCodeManager;
import org.slf4j.LoggerFactory;

/**
 * 環境定義リソースに定義されている内容を元にHibernate(4.X系)を利用したデータベース処理アドオンモジュールの初期化を行うインタフェースを提供します。<br>
 * <p>
 * 環境定義に当アドオンモジュールの初期化クラスが指定されている場合に{@link org.phosphoresce.webcore.core.GlobalConfiguration}より呼び出され、初期化処理が実行されます。<br>
 * クラス概要はコアモジュール側パッケージドキュメントを参照してください。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2011/12/24	Kitagawa		新規作成
 *-->
 */
public class HibernateConfiguration implements HibernateConstants {

	/** ロガーオブジェクト */
	private static CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(HibernateConfiguration.class));

	/**
	 * コンストラクタ<br>
	 */
	private HibernateConfiguration() {
		super();
	}

	/**
	 * Hibernateアドオンの初期化処理を行います。<br>
	 * 当初期化処理はアプリケーション共通初期化処理が完了されていることが前提となります。<br>
	 */
	public synchronized static void configure() {
		try {
			log.output("FHBN00001");

			configureEvironment();
			configureSCodeAutoload();
			configureSCodeManager();

			log.output("FHBN00002");
		} catch (Throwable e) {
			throw new WebcoreConfigurationException("FHBN00006", e);
		}
	}

	/**
	 * Hibernateライブラリ環境初期化処理を行います。<br>
	 * 当初期化処理はアプリケーション共通初期化処理が完了されていることが前提となります。<br>
	 */
	private synchronized static void configureEvironment() {
		// ディフォルト定義分初期化処理
		configureEvironment(null);
		log.output("FHBN00004");

		// 名前空間定義分初期化処理
		String[] namespaces = Config.getStrings(HibernateConfigName.HIBERNATE_NAMESPACES);
		for (String namespace : namespaces) {
			configureEvironment(namespace);
			log.output("FHBN00005", namespace);
		}
	}

	/**
	 * Hibernateライブラリ環境初期化処理を行います。<br>
	 * 当初期化処理はアプリケーション共通初期化処理が完了されていることが前提となります。<br>
	 * @param namespace 名前空間文字列
	 */
	private synchronized static void configureEvironment(String namespace) {
		try {
			String suffix = "";
			if (!StringUtil.isEmpty(namespace)) {
				suffix += "." + namespace;
			}

			/*
			 * 基底環境設定定義ロード
			 */
			String setting = Config.getString(HibernateConfigName.HIBERNATE_CONFIG.suffixedName(suffix));
			Configuration configuration = new Configuration();
			if (setting.toLowerCase().endsWith(CONFIG_XML_EXTENSION)) {
				Document document = DOM4JUtil.loadByDOM4J(setting);
				configuration.configure(DOM4JUtil.convertW3C(GlobalEnvironment.bindEnv(document)));
			} else if (setting.toLowerCase().endsWith(CONFIG_PROPERTIES_EXTENSION)) {
				configuration.setProperties(GlobalEnvironment.bindEnv(PropertiesUtil.load(setting)));
			}

			/*
			 * アノテーションモデルクラス追加
			 */
			for (String modelClass : Config.getString(HibernateConfigName.HIBERNATE_ANNOTATION_CLASSES.suffixedName(suffix)).split(",")) {
				if (!StringUtil.isEmpty(modelClass) && !modelClass.trim().startsWith("#")) {
					configuration.addAnnotatedClass(Class.forName(modelClass.trim()));
					log.output("FHBN00023", modelClass.trim());
				}
			}

			/*
			 * トランザクションファクトリクラス登録
			 */
			if (!StringUtil.isEmpty(namespace)) {
				TransactionFactory.initialize(namespace, configuration);
			} else {
				TransactionFactory.initialize(configuration);
			}
		} catch (Throwable e) {
			throw new HibernateConfigureException("FHBN00007", e);
		}
	}

	/**
	 * システムコードマスタの自動登録処理を行います。<br>
	 * @throws Throwable 処理中に予期せぬエラーが発生した場合にスローされます 
	 */
	private synchronized static void configureSCodeAutoload() throws Throwable {
		HibernateTransactionManager transaction = null;
		HibernateTransactionContext<SUser> context = null;
		SystemMasterService service = null;

		/*
		 * 自動ロード設定が行われていない場合は処理スキップ
		 */
		if (!Config.getBoolean(HibernateConfigName.HIBERNATE_CODEMASTER_AUTOLOAD)) {
			log.output("FHBN00027");
			return;
		}

		try {
			transaction = new HibernateTransactionManager();
			context = new HibernateTransactionContext<SUser>(transaction, null, new Date());
			service = new SystemMasterService(context);
			transaction.open();

			/*
			 * 未登録又はExcelファイル更新日が新しい場合は登録を行う
			 */
			SConfig sConfig = service.getSConfigByProperty(SCONFIG_KEY_SCODE_LOAD_TIMESTAMP);
			SimpleDateFormat timestampFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
			long excelTimestamp = new File(Config.getString(HibernateConfigName.HIBERNATE_CODEMASTER_PATH)).lastModified();
			boolean load = false;
			if (sConfig == null) {
				log.output("FHBN00029");
				load = true;
			} else {
				String lastmodified = sConfig.getValue1();
				long registedTimestamp = timestampFormat.parse(lastmodified).getTime();
				if (registedTimestamp < excelTimestamp) {
					log.output("FHBN00030");
					load = true;
				} else {
					log.output("FHBN00028");
					load = false;
				}
			}
			if (load) {
				service.removeAllSCode();
				PoiWorkbook workbook = new PoiWorkbook(PoiWorkbookType.EXCEL2003, Config.getString(HibernateConfigName.HIBERNATE_CODEMASTER_PATH));
				PoiSheet sheet = workbook.getSheet(0);
				for (int i = 1; i <= sheet.getLastRowNum(); i++) {
					String category = sheet.getCell(i, 0).getString();
					String code = sheet.getCell(i, 1).getString();
					String orderNo = sheet.getCell(i, 2).getString();
					String caption = sheet.getCell(i, 3).getString();
					String shortCaption = sheet.getCell(i, 4).getString();
					String definition1 = sheet.getCell(i, 5).getString();
					String definition2 = sheet.getCell(i, 6).getString();
					String definition3 = sheet.getCell(i, 7).getString();
					String definition4 = sheet.getCell(i, 8).getString();
					String definition5 = sheet.getCell(i, 9).getString();
					if (StringUtil.isEmpty(category)) {
						continue;
					}
					SCode sCode = new SCode();
					sCode.setCategory(category);
					sCode.setCode(code);
					sCode.setOrderNo(Integer.parseInt(orderNo));
					sCode.setCaption(caption);
					sCode.setShortCaption(shortCaption);
					sCode.setDefinition1(definition1);
					sCode.setDefinition2(definition2);
					sCode.setDefinition3(definition3);
					sCode.setDefinition4(definition4);
					sCode.setDefinition5(definition5);
					service.registSCode(sCode);
					log.output("FHBN00031", category, code, caption);
				}
				if (sConfig == null) {
					sConfig = new SConfig();
					sConfig.setProperty(SCONFIG_KEY_SCODE_LOAD_TIMESTAMP);
				}
				sConfig.setValue1(timestampFormat.format(new Date(excelTimestamp)));
				service.registSConfig(sConfig);
			}
		} catch (Throwable e) {
			throw e;
		} finally {
			if (transaction != null) {
				transaction.commit();
				transaction.close();
			}
		}
	}

	/**
	 * システムコードマスタ管理クラスの初期化を行います。<br>
	 * @throws Throwable 処理中に予期せぬエラーが発生した場合にスローされます 
	 */
	@SuppressWarnings("deprecation")
	private synchronized static void configureSCodeManager() throws Throwable {
		HibernateTransactionManager transaction = null;
		HibernateTransactionContext<SUser> context = null;
		SystemMasterService service = null;
		try {
			transaction = new HibernateTransactionManager();
			context = new HibernateTransactionContext<SUser>(transaction, null, new Date());
			service = new SystemMasterService(context);
			transaction.open();

			List<SCode> sCodes = service.getAllSCode();
			SCodeManager.initialize(sCodes);
		} catch (Throwable e) {
			throw e;
		} finally {
			if (transaction != null) {
				transaction.rollback();
				transaction.close();
			}
		}
	}

	/**
	 * Hibernateアドオンの破棄処理を実行します。<br>
	 * @throws Throwable 終了処理中に予期せぬエラーが発生した場合にスローされます 
	 */
	public synchronized static void destroy() throws Throwable {
		JODSOfficeProcessManager.destroy();
		log.output("FHBN00003");
	}
}
