package org.phosphoresce.webcore.ext.hibernate4.model.usertype;

import java.io.Serializable;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.hibernate.HibernateException;
import org.hibernate.engine.spi.SessionImplementor;
import org.hibernate.usertype.UserType;

/**
 * Hibernateエンティティで利用するユーザータイプ定数の共通インタフェースを提供します。<br>
 * <p>
 * エンティティフィールドにおいて、定数を扱う場合、文字列等の直接的な値を保持せず、このクラスを継承して提供されるタイプクラスを利用します。<br>
 * 作成するタイプクラスは必ず、このクラスが提供するコードをシグネチャに持つコンストラクタ、及び、引数を持たないディフォルトコンストラクタを設ける必要があります。<br>
 * また、抽象メソッドとして提供されている{@link #allTypes()}をタイプクラスで実装し、タイプクラスが提供する全ての定義を返却する実装を行います。<br>
 * エンティティに対して設定されたタイプは実際のデータベース上にタイプクラスが提供するコード文字列として保存されます。<br>
 * </p>
 * <p>
 * タイプクラスの実装例は以下の通りで、public static finalなフィールドの集合体として作成されます。<br>
 * <pre>
 * public class SampleType extends AbstractUserType<SampleType> {
 * 
 *    // コード0として保存されるタイプ
 *    public static final SampleType FOO = new SampleType("0");
 * 
 *    // コード1として保存されるタイプ
 *    public static final SampleType BAR = new SampleType("1");
 * 
 *    // タイプクラスが提供する全ての定義を返却する
 *    @Override
 *    public SampleType[] allTypes() {
 *        return new SampleType[]{FOO, BAR};
 *    }
 * 
 *    // 引数を持たないディフォルトコンストラクタ
 *    public SampleType() {
 *        super();
 *    }
 * 
 *    // コードを引数とするコンストラクタ
 *    protected SampleType(String code) {
 *        super(code);
 *    }
 * }
 * </pre>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/07/25	Kitagawa		新規作成
 *-->
 */
public abstract class AbstractUserType<T extends AbstractUserType<T>> implements UserType, Serializable {

	/** タイプコード */
	private String code;

	/**
	 * コンストラクタ<br>
	 */
	public AbstractUserType() {
		super();
	}

	/**
	 * コンストラクタ<br>
	 * @param code タイプコード
	 */
	protected AbstractUserType(String code) {
		super();
		this.code = code;
	}

	/**
	 * 全てのタイプを提供します。<br>
	 * @return 全てのタイプ
	 */
	public abstract T[] allTypes();

	/**
	 * タイプコードを提供します。<br>
	 * @return タイプコード
	 */
	public final String code() {
		return code;
	}

	/**
	 * 指定されたタイプコードのタイプインスタンスを提供します。<br>
	 * 指定されたタイプコードに対応するものがない場合はnullを返却します。<br>
	 * @param code タイプコード
	 * @return タイプインスタンス
	 */
	public final T valueOf(String code) {
		for (T t : allTypes()) {
			if (t.code().equals(code)) {
				return t;
			}
		}
		return null;
	}

	/**
	 * Reconstruct an object from the cacheable representation.<br>
	 * At the very least this method should perform a deep copy if the type is mutable.<br>
	 * (optional operation)<br>
	 * @param cached the object to be cached
	 * @param owner the owner of the cached object
	 * @return a reconstructed object from the cachable representation
	 * @see org.hibernate.usertype.UserType#assemble(java.io.Serializable, java.lang.Object)
	 */
	@Override
	public final Object assemble(Serializable cached, Object owner) throws HibernateException {
		return cached;
	}

	/**
	 * Return a deep copy of the persistent state, stopping at entities and at collections.<br>
	 * It is not necessary to copy immutable objects, or null values, in which case it is safe to simply return the argument.
	 * @param value the object to be cloned, which may be null
	 * @return Object a copy
	 * @see org.hibernate.usertype.UserType#deepCopy(java.lang.Object)
	 */
	@Override
	public final Object deepCopy(Object value) throws HibernateException {
		return value;
	}

	/**
	 * Transform the object into its cacheable representation.<br>
	 * At the very least this method should perform a deep copy if the type is mutable.<br>
	 * That may not be enough for some implementations, however; for example, associations must be cached as identifier values.<br>
	 * (optional operation)<br>
	 * @param value the object to be cached
	 * @return a cachable representation of the object
	 * @see org.hibernate.usertype.UserType#disassemble(java.lang.Object)
	 */
	@Override
	public final Serializable disassemble(Object value) throws HibernateException {
		return (Serializable) value;
	}

	/**
	 * Compare two instances of the class mapped by this type for persistence "equality".<br>
	 * Equality of the persistent state.<br>
	 * @param object1 compare object
	 * @param object2 compare object
	 * @return if euals object, reuturn true
	 * @see org.hibernate.usertype.UserType#equals(java.lang.Object, java.lang.Object)
	 */
	@Override
	public final boolean equals(Object object1, Object object2) throws HibernateException {
		if (object1 == object2) {
			return true;
		}
		if (null == object1 || null == object2) {
			return false;
		}
		return object1.equals(object2);
	}

	/**
	 * Get a hashcode for the instance, consistent with persistence "equality".<br>
	 * @param object target object
	 * @return hash code
	 * @see org.hibernate.usertype.UserType#hashCode(java.lang.Object)
	 */
	@Override
	public final int hashCode(Object object) throws HibernateException {
		return object.hashCode();
	}

	/**
	 * Are objects of this type mutable?<br>
	 * @return if object is mutable, return true
	 * @see org.hibernate.usertype.UserType#isMutable()
	 */
	@Override
	public final boolean isMutable() {
		return false;
	}

	/**
	 * Retrieve an instance of the mapped class from a JDBC resultset.<br>
	 * Implementors should handle possibility of null values.<br>
	 * @param resultSet a JDBC result set
	 * @param names the column names
	 * @param owner the containing entity
	 * @return Object
	 * @see org.hibernate.usertype.UserType#nullSafeGet(java.sql.ResultSet, java.lang.String[], org.hibernate.engine.spi.SessionImplementor, java.lang.Object)
	 */
	@Override
	public final Object nullSafeGet(ResultSet resultSet, String[] names, SessionImplementor arg2, Object owner) throws HibernateException, SQLException {
		String name = resultSet.getString(names[0]);
		T result = null;
		if (!resultSet.wasNull()) {
			result = valueOf(name);
		}
		return result;
	}

	/**
	 * Write an instance of the mapped class to a prepared statement.<br>
	 * Implementors should handle possibility of null values.<br>
	 * A multi-column type should be written to parameters starting from index.<br>
	 * @param preparedStatement a JDBC prepared statement
	 * @param value the object to write
	 * @param index statement parameter index
	 * @param sessionImplementor SessionImplementor instance
	 * @see org.hibernate.usertype.UserType#nullSafeSet(java.sql.PreparedStatement, java.lang.Object, int, org.hibernate.engine.spi.SessionImplementor)
	 */
	@SuppressWarnings("unchecked")
	@Override
	public final void nullSafeSet(PreparedStatement preparedStatement, Object value, int index, SessionImplementor sessionImplementor) throws HibernateException, SQLException {
		if (null == value) {
			preparedStatement.setNull(index, Types.VARCHAR);
		} else {
			preparedStatement.setString(index, ((T) value).code());
		}
	}

	/**
	 * During merge, replace the existing (target) value in the entity we are merging to with a new (original) value
	 * from the detached entity we are merging. For immutable objects, or null values, it is safe to simply return the first parameter.<br>
	 * For mutable objects, it is safe to return a copy of the first parameter.<br>
	 * For objects with component values, it might make sense to recursively replace component values.<br>
	 * @param original the value from the detached entity being merged
	 * @param target the value in the managed entity
	 * @return the value to be merged
	 * @see org.hibernate.usertype.UserType#replace(java.lang.Object, java.lang.Object, java.lang.Object)
	 */
	@Override
	public final Object replace(Object original, Object target, Object owner) throws HibernateException {
		return original;
	}

	/**
	 * The class returned by nullSafeGet().<br>
	 * @return Class
	 * @see org.hibernate.usertype.UserType#returnedClass()
	 */
	@SuppressWarnings("unchecked")
	@Override
	public final Class<T> returnedClass() {
		return (Class<T>) getClass();
	}

	/**
	 * Return the SQL type codes for the columns mapped by this type.<br>
	 * The codes are defined on java.sql.Types.<br>
	 * @return int[] the typecodes
	 * @see org.hibernate.usertype.UserType#sqlTypes()
	 */
	@Override
	public final int[] sqlTypes() {
		return new int[] { Types.VARCHAR };
	}
}
