package org.phosphoresce.webcore.ext.hibernate4.transaction;

import static org.hibernate.criterion.Restrictions.*;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.criterion.Order;
import org.phosphoresce.lib.commons.util.ReflectionUtil;
import org.phosphoresce.webcore.ext.hibernate4.util.SCodeManager;

/**
 * 任意のエンティティに対するHibernateデータベースアクセス処理クラスの共通的なインタフェースを提供します。<br>
 * <p>
 * このクラスが提供するインタフェースはそれぞれのメソッドで利用するエンティティクラスを指定します。<br>
 * アプリケーション開発時に作成するデータベースアクセスオブジェクトはこのクラスではなく、{@link AbstractGenericHibernateDAO}を継承して作成します。<br>
 * このクラスは、汎用的なデータベースアクセスオブジェクトを作成する際に、総称型によるエンティティ制限を設けることが出来ない為に設けられました。<br>
 * </p>
 *
 * @param <E> データベースアクセス対象エンティティクラス
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/08/16	Kitagawa		新規作成
 *-->
 */
abstract class AbstractHibernateDAO {

	/** トランザクション管理オブジェクト */
	HibernateTransactionManager transaction;

	/**
	 * コンストラクタ<br>
	 * @param transaction トランザクション管理オブジェクト
	 */
	AbstractHibernateDAO(HibernateTransactionManager transaction) {
		if (transaction == null) {
			throw new NullPointerException();
		}
		this.transaction = transaction;
	}

	/**
	 * データベースクエリ条件オブジェクトを生成します。<br>
	 * @param entityClass 対象エンティティクラス
	 * @return データベースクエリ条件オブジェクト
	 */
	<C extends AbstractHibernateEntity> GenericCriteria<C> createCriteria(Class<C> entityClass) {
		Session session = transaction.getSession();
		Criteria criteria = session.createCriteria(entityClass);
		return new GenericCriteria<C>(criteria);
	}

	/**
	 * データベースクエリ条件オブジェクトを生成します。<br>
	 * 対象エンティティが削除状態を保持するエンティティの場合、削除状態のエンティティを除外する条件を持ちます。<br>
	 * @param entityClass 対象エンティティクラス
	 * @return データベースクエリ条件オブジェクト
	 */
	<C extends AbstractHibernateEntity> GenericCriteria<C> createCriteriaExcludeDeleted(Class<C> entityClass) {
		GenericCriteria<C> criteria = createCriteria(entityClass);
		if (ReflectionUtil.getSuperclasses(entityClass).contains(AbstractHibernateDefaultEntity.class)) {
			criteria.add(ne("deleteType", SCodeManager.getCode("S0001", "1")));
		}
		return criteria;
	}

	/**
	 * SQLデータベースクエリを生成します。<br>
	 * DAOクラスにおいて当インタフェースを利用した場合、エンティティクラスで制限がかからないクエリとなることに注意して下さい。<br>
	 * これは利用者がDAOクラスの位置づけ(エンティティ対応しているDAO)に注意して利用する必要があり、場合によってはアプリケーション開発規約で利用禁止とすることも検討して下さい。<br>
	 * @param sql SQLデータベースクエリ
	 * @return SQLデータベースクエリオブジェクト
	 */
	SQLQuery createSQLQuery(String sql) {
		return transaction.getSession().createSQLQuery(sql);
	}

	/**
	 * HQLデータベースクエリを生成します。<br>
	 * DAOクラスにおいて当インタフェースを利用した場合、エンティティクラスで制限がかからないクエリとなることに注意して下さい。<br>
	 * これは利用者がDAOクラスの位置づけ(エンティティ対応しているDAO)に注意して利用する必要があり、場合によってはアプリケーション開発規約で利用禁止とすることも検討して下さい。<br>
	 * @param hql HQLデータベースクエリ
	 * @return HQLデータベースクエリオブジェクト
	 */
	Query createHQLQuery(String hql) {
		return transaction.getSession().createQuery(hql);
	}

	/**
	 * 指定されたレコードIDでエンティティを検索します。<br>
	 * 当インタフェースは削除済の条件は加味されません。<br>
	 * @param entityClass 対象エンティティクラス
	 * @param id レコードID
	 * @return エンティティオブジェクト
	 */
	<C extends AbstractHibernateEntity> C findById(Class<C> entityClass, Integer id) {
		GenericCriteria<C> criteria = createCriteria(entityClass);
		criteria.add(idEq(id));
		return criteria.uniqueResult();
	}

	/**
	 * データベースに登録されている全てのエンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @param orders ソート条件オブジェクト
	 * @return エンティティレコードリスト
	 */
	<C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass, boolean includeDeleted, List<Order> orders) {
		GenericCriteria<C> criteria = null;
		if (includeDeleted) {
			criteria = createCriteria(entityClass);
		} else {
			criteria = createCriteriaExcludeDeleted(entityClass);
		}
		if (orders != null) {
			for (Order order : orders) {
				criteria.addOrder(order);
			}
		}
		return criteria.list();
	}

	/**
	 * データベースに登録されている全ての未削除エンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @param orders ソート条件オブジェクト
	 * @return エンティティレコードリスト
	 */
	<C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass, List<Order> orders) {
		return findAll(entityClass, false, orders);
	}

	/**
	 * データベースに登録されている全てのエンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @return エンティティレコードリスト
	 */
	<C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass, boolean includeDeleted) {
		return findAll(entityClass, includeDeleted, null);
	}

	/**
	 * データベースに登録されている全ての未削除エンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @return エンティティレコードリスト
	 */
	<C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass) {
		return findAll(entityClass, false);
	}

	/**
	 * データベースに登録されているエンティティレコード数を取得します。<br>
	 * @param entityClass 対象エンティティクラス
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @return エンティティレコード数
	 */
	<C extends AbstractHibernateEntity> long countAll(Class<C> entityClass, boolean includeDeleted) {
		GenericCriteria<C> criteria = null;
		if (includeDeleted) {
			criteria = createCriteria(entityClass);
		} else {
			criteria = createCriteriaExcludeDeleted(entityClass);
		}
		return criteria.count();
	}

	/**
	 * データベースに登録されている未削除のエンティティレコード数を取得します。<br>
	 * @param entityClass 対象エンティティクラス
	 * @return エンティティレコード数
	 */
	<C extends AbstractHibernateEntity> long countAll(Class<C> entityClass) {
		return countAll(entityClass, false);
	}
}
