package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.lang.reflect.Constructor;
import java.util.List;

import org.hibernate.LockMode;
import org.hibernate.LockOptions;
import org.hibernate.PessimisticLockException;
import org.hibernate.dialect.lock.LockingStrategyException;
import org.phosphoresce.webcore.core.transaction.TransactionService;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateEntityLockException;
import org.phosphoresce.webcore.ext.hibernate4.exception.HibernateTransactionException;
import org.phosphoresce.webcore.ext.hibernate4.model.SUser;
import org.phosphoresce.webcore.ext.hibernate4.util.SCodeManager;

/**
 * Hibernateエンティティモデルに対する各種データベース処理共通インタフェースを提供します。<br>
 * <p>
 * アドオンモジュールを利用したデータベース処理を包括的に提供するサービスクラスをこのクラスを継承して実装します。<br>
 * サービスクラスでは、各メソッドにおいて必要なデータベースアクセスオブジェクトを生成し、メソッドパラメータで指定した内容を元にデータベーステーブルエンティティに対する検索、更新を行います。<br>
 * </p>
 * <p>
 * このクラスを継承して実装されるサービスクラスは基本的に、このクラスが提供されるコンストラクタと同様のコンストラクタが提供されるクラス設計とする必要があることに注意してください。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/02/01	Kitagawa		新規作成
 *-->
 */
public abstract class AbstractHibernateService<U extends SUser> extends TransactionService<HibernateTransactionManager, U, HibernateTransactionContext<U>> {

	/**
	 * コンストラクタ<br>
	 * @param transactionContext トランザクション処理コンテキストオブジェクト
	 */
	public AbstractHibernateService(HibernateTransactionContext<U> transactionContext) {
		super(transactionContext);
	}

	/**
	 * データベースアクセスオブジェクトクラスインスタンスを生成します。<br>
	 * 生成されたインスタンスでのデータベースアクセスはロック処理が行われません。<br>
	 * 更新中のレコードに対して取得処理を行う場合、更新前のレコードキャッシュから取得が行われます。<br>
	 * @param clazz データベースアクセスオブジェクトクラス
	 * @return クラスインスタンス
	 */
	protected final <D extends AbstractHibernateDAO> D createDAO(Class<D> clazz) {
		try {
			Constructor<D> constructor = clazz.getConstructor(new Class[] { HibernateTransactionManager.class });
			D instance = constructor.newInstance(new Object[] { getTransactionContext().getTransaction() });
			return instance;
		} catch (Throwable e) {
			throw new HibernateTransactionException("FHBN00012", e);
		}
	}

	/**
	 * エンティティに対して楽観的排他を実施します。<br>
	 * @param entity エンティティオブジェクト
	 */
	protected final void lockOptimistic(AbstractHibernateEntity entity) {
		try {
			getTransactionContext().getTransaction().getSession().buildLockRequest(LockOptions.NONE).setLockMode(LockMode.OPTIMISTIC).lock(entity);
		} catch (LockingStrategyException e) {
			throw new HibernateEntityLockException(e);
		} catch (PessimisticLockException e) {
			throw new HibernateEntityLockException(e);
		}
	}

	/**
	 * エンティティに対して悲観的排他を実施します。<br>
	 * @param entity エンティティオブジェクト
	 */
	protected final void lockPessimistic(AbstractHibernateEntity entity) {
		try {
			getTransactionContext().getTransaction().getSession().buildLockRequest(LockOptions.NONE).setLockMode(LockMode.PESSIMISTIC_WRITE).lock(entity);
		} catch (LockingStrategyException e) {
			throw new HibernateEntityLockException(e);
		} catch (PessimisticLockException e) {
			throw new HibernateEntityLockException(e);
		}
	}

	/**
	 * エンティティに対して悲観的排他を実施します。<br>
	 * @param entity エンティティオブジェクト
	 */
	protected final void lockPessimisticNoWait(AbstractHibernateEntity entity) {
		try {
			getTransactionContext().getTransaction().getSession().buildLockRequest(LockOptions.NONE).setLockMode(LockMode.UPGRADE_NOWAIT).lock(entity);
		} catch (LockingStrategyException e) {
			throw new HibernateEntityLockException(e);
		} catch (PessimisticLockException e) {
			throw new HibernateEntityLockException(e);
		}
	}

	/**
	 * エンティティを保存します。<br>
	 * @param entity エンティティオブジェクト
	 */
	protected final void save(AbstractHibernateEntity entity) {
		if (entity != null) {
			if (entity instanceof AbstractHibernateDefaultEntity) {
				AbstractHibernateDefaultEntity defaultEntity = (AbstractHibernateDefaultEntity) entity;
				if (defaultEntity.getEntryDate() == null) {
					defaultEntity.setEntryUser(getTransactionContext().getProcessUser());
					defaultEntity.setEntryDate(getTransactionContext().getProcessDate());
				}
				defaultEntity.setEntryDate(getTransactionContext().getProcessDate());
				defaultEntity.setModifyUser(getTransactionContext().getProcessUser());
				defaultEntity.setModifyDate(getTransactionContext().getProcessDate());
				//defaultEntity.setDeleteType(DeleteType.VALIDATE);
				defaultEntity.setDeleteType(SCodeManager.getCode("S0001", "0"));
			}
			entity.validate();
			getTransactionContext().getTransaction().getSession().saveOrUpdate(entity);
		}
	}

	/**
	 * エンティティを保存します。<br>
	 * @param entities エンティティオブジェクト
	 */
	protected final <E extends AbstractHibernateEntity> void save(List<E> entities) {
		if (entities != null) {
			for (AbstractHibernateEntity entity : entities) {
				save(entity);
			}
		}
	}

	/**
	 * エンティティを論理削除します。<br>
	 * @param entity エンティティオブジェクト
	 */
	protected final void delete(AbstractHibernateEntity entity) {
		if (entity != null) {
			if (entity instanceof AbstractHibernateDefaultEntity) {
				AbstractHibernateDefaultEntity defaultEntity = (AbstractHibernateDefaultEntity) entity;
				if (defaultEntity.getEntryDate() == null) {
					defaultEntity.setEntryUser(getTransactionContext().getProcessUser());
					defaultEntity.setEntryDate(getTransactionContext().getProcessDate());
				}
				defaultEntity.setModifyUser(getTransactionContext().getProcessUser());
				defaultEntity.setModifyDate(getTransactionContext().getProcessDate());
				defaultEntity.setDeleteUser(getTransactionContext().getProcessUser());
				defaultEntity.setDeleteDate(getTransactionContext().getProcessDate());
				//defaultEntity.setDeleteType(DeleteType.DELETED);
				defaultEntity.setDeleteType(SCodeManager.getCode("S0001", "1"));
			}
			entity.validate();
			getTransactionContext().getTransaction().getSession().saveOrUpdate(entity);
		}
	}

	/**
	 * エンティティを論理削除します。<br>
	 * @param entities エンティティオブジェクト
	 */
	protected final <E extends AbstractHibernateEntity> void delete(List<E> entities) {
		if (entities != null) {
			for (AbstractHibernateEntity entity : entities) {
				delete(entity);
			}
		}
	}

	/**
	 * エンティティを物理削除します。<br>
	 * @param entity エンティティオブジェクト
	 */
	protected final void remove(AbstractHibernateEntity entity) {
		if (entity != null) {
			getTransactionContext().getTransaction().getSession().delete(entity);
		}
	}

	/**
	 * エンティティを論理削除します。<br>
	 * @param entities エンティティオブジェクト
	 */
	protected final <E extends AbstractHibernateEntity> void remove(List<E> entities) {
		if (entities != null) {
			for (AbstractHibernateEntity entity : entities) {
				remove(entity);
			}
		}
	}
}
