package org.phosphoresce.webcore.ext.hibernate4.transaction;

import java.util.List;

import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.criterion.Order;

/**
 * エンティティに依存しない汎用的なデータベースアクセスを行う為のインタフェースを提供します。<br>
 * <p>
 * アプリケーション規模に応じた方式設計で、ロジッククラス、サービスクラス、DAOクラスの階層とした場合に不要にクラスリソースが増加する可能性があります。<br>
 * その場合、当クラスをアプリケーションごとの共通サービスクラスにおいてデータベースアクセスのラッパークラスインスタンスとして提供することで、
 * サービスクラスから汎用的なDAOクラスのインタフェースを利用することが可能となります。<br>
 * </p>
 * <p>
 * 大規模なアプリケーションの場合、サービスクラスでは、DAOクラスが提供する粒度の細かい業務処理依存のない処理インタフェースはの集合体となりますが、
 * 小規模なアプリケーションの場合、多くはDAOクラスが提供するインタフェースに近いサービスクラスが大量に発生する可能性があります。<br>
 * このクラスを利用することでエンティティ毎にDAOクラスを設ける必要がなくなり、冗長的なサービスクラスとDAOクラスの関係がなくなることを意味します。<br>
 * 
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/08/16	Kitagawa		新規作成
 *-->
 */
public final class GeneralHibernateDAO extends AbstractHibernateDAO {

	/**
	 * コンストラクタ<br>
	 * @param transaction トランザクション管理オブジェクト
	 */
	public GeneralHibernateDAO(HibernateTransactionManager transaction) {
		super(transaction);
	}

	/**
	 * データベースクエリ条件オブジェクトを生成します。<br>
	 * @param entityClass 対象エンティティクラス
	 * @return データベースクエリ条件オブジェクト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#createCriteria(java.lang.Class)
	 */
	@Override
	public <C extends AbstractHibernateEntity> GenericCriteria<C> createCriteria(Class<C> entityClass) {
		return super.createCriteria(entityClass);
	}

	/**
	 * データベースクエリ条件オブジェクトを生成します。<br>
	 * 対象エンティティが削除状態を保持するエンティティの場合、削除状態のエンティティを除外する条件を持ちます。<br>
	 * @param entityClass 対象エンティティクラス
	 * @return データベースクエリ条件オブジェクト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#createCriteriaExcludeDeleted(java.lang.Class)
	 */
	@Override
	public <C extends AbstractHibernateEntity> GenericCriteria<C> createCriteriaExcludeDeleted(Class<C> entityClass) {
		return super.createCriteriaExcludeDeleted(entityClass);
	}

	/**
	 * SQLデータベースクエリを生成します。<br>
	 * DAOクラスにおいて当インタフェースを利用した場合、エンティティクラスで制限がかからないクエリとなることに注意して下さい。<br>
	 * これは利用者がDAOクラスの位置づけ(エンティティ対応しているDAO)に注意して利用する必要があり、場合によってはアプリケーション開発規約で利用禁止とすることも検討して下さい。<br>
	 * @param sql SQLデータベースクエリ
	 * @return SQLデータベースクエリオブジェクト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#createSQLQuery(java.lang.String)
	 */
	@Override
	public SQLQuery createSQLQuery(String sql) {
		return super.createSQLQuery(sql);
	}

	/**
	 * HQLデータベースクエリを生成します。<br>
	 * DAOクラスにおいて当インタフェースを利用した場合、エンティティクラスで制限がかからないクエリとなることに注意して下さい。<br>
	 * これは利用者がDAOクラスの位置づけ(エンティティ対応しているDAO)に注意して利用する必要があり、場合によってはアプリケーション開発規約で利用禁止とすることも検討して下さい。<br>
	 * @param hql HQLデータベースクエリ
	 * @return HQLデータベースクエリオブジェクト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#createHQLQuery(java.lang.String)
	 */
	@Override
	public Query createHQLQuery(String hql) {
		return super.createHQLQuery(hql);
	}

	/**
	 * 指定されたレコードIDでエンティティを検索します。<br>
	 * 当インタフェースは削除済の条件は加味されません。<br>
	 * @param entityClass 対象エンティティクラス
	 * @param id レコードID
	 * @return エンティティオブジェクト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#findById(java.lang.Class, java.lang.Integer)
	 */
	@Override
	public <C extends AbstractHibernateEntity> C findById(Class<C> entityClass, Integer id) {
		return super.findById(entityClass, id);
	}

	/**
	 * データベースに登録されている全てのエンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @param orders ソート条件オブジェクト
	 * @return エンティティレコードリスト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#findAll(java.lang.Class, boolean, java.util.List)
	 */
	@Override
	public <C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass, boolean includeDeleted, List<Order> orders) {
		return super.findAll(entityClass, includeDeleted, orders);
	}

	/**
	 * データベースに登録されている全ての未削除エンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @param orders ソート条件オブジェクト
	 * @return エンティティレコードリスト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#findAll(java.lang.Class, java.util.List)
	 */
	@Override
	public <C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass, List<Order> orders) {
		return super.findAll(entityClass, orders);
	}

	/**
	 * データベースに登録されている全てのエンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @return エンティティレコードリスト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#findAll(java.lang.Class, boolean)
	 */
	@Override
	<C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass, boolean includeDeleted) {
		return super.findAll(entityClass, includeDeleted);
	}

	/**
	 * データベースに登録されている全ての未削除エンティティレコードを取得します。<br>
	 * マスタデータ等で利用されることはありますが、通常のトランザクションデータ等では利用されることは想定されません。
	 * @param entityClass 対象エンティティクラス
	 * @return エンティティレコードリスト
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#findAll(java.lang.Class)
	 */
	@Override
	public <C extends AbstractHibernateEntity> List<C> findAll(Class<C> entityClass) {
		return super.findAll(entityClass);
	}

	/**
	 * データベースに登録されているエンティティレコード数を取得します。<br>
	 * @param entityClass 対象エンティティクラス
	 * @param includeDeleted 論理削除済みデータを含める場合にtrueを指定
	 * @return エンティティレコード数
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#countAll(java.lang.Class, boolean)
	 */
	@Override
	public <C extends AbstractHibernateEntity> long countAll(Class<C> entityClass, boolean includeDeleted) {
		return super.countAll(entityClass, includeDeleted);
	}

	/**
	 * データベースに登録されている未削除のエンティティレコード数を取得します。<br>
	 * @param entityClass 対象エンティティクラス
	 * @return エンティティレコード数
	 * @see org.phosphoresce.webcore.ext.hibernate4.transaction.AbstractHibernateDAO#countAll(java.lang.Class)
	 */
	@Override
	public <C extends AbstractHibernateEntity> long countAll(Class<C> entityClass) {
		return super.countAll(entityClass);
	}
}
