package org.phosphoresce.webcore.ext.hibernate4.util;

import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.phosphoresce.webcore.ext.hibernate4.model.SCode;
import org.slf4j.LoggerFactory;

/**
 * データベースに登録されているシステムコード情報をオンメモリで管理するインタフェースを提供します。<br>
 * <p>
 * 当クラスはシステムコードマスタエンティティ({@link org.phosphoresce.webcore.ext.hibernate4.model.SCode})をアプリケーション起動中、永続的に保持、管理します。<br>
 * 当クラスはアプリケーション起動時に初期化されることを前提としたクラスで、初期化後にユーザーが個別に操作することはありません。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/07/31	Kitagawa		新規作成
 *-->
 */
public class SCodeManager {

	/** ロガーオブジェクト */
	private static CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(SCodeManager.class));

	/** クラスインスタンス */
	private static SCodeManager instance;

	/** システムマスタデータ */
	private Map<String, Map<String, SCode>> data;

	/**
	 * コンストラクタ<br>
	 */
	private SCodeManager() {
		super();
	}

	/**
	 * クラスインスタンスを取得します。<br>
	 * @return クラスインスタンス
	 */
	private static SCodeManager instance() {
		if (instance == null) {
			instance = new SCodeManager();
		}
		return instance;
	}

	/**
	 * 指定されたシステムマスタレコードリストからクラスを初期化します。<br>
	 * @param sCodes システムマスタレコードリスト
	 * @deprecated 起動時に初期化されることを前提としたクラスで、初期化後にユーザーが個別に操作することはありません。
	 */
	@Deprecated
	public static void initialize(List<SCode> sCodes) {
		SCodeManager.instance().data = new HashMap<String, Map<String, SCode>>();
		synchronized (SCodeManager.instance().data) {
			if (sCodes != null) {
				for (SCode sCode : sCodes) {
					if (sCode == null) {
						continue;
					}
					String category = sCode.getCategory();
					String code = sCode.getCode();
					if (!SCodeManager.instance().data.containsKey(category)) {
						SCodeManager.instance().data.put(category, new HashMap<String, SCode>());
					}

					Map<String, SCode> categorizedData = SCodeManager.instance().data.get(category);
					categorizedData.put(code, sCode);
				}
			}
		}
		log.output("FHBN00026");
	}

	/**
	 * 指定されたカテゴリとコードからシステムコードマスタ情報を取得します。<br>
	 * @param category カテゴリコード
	 * @param code コード
	 * @return システムコードマスタエンティティ
	 */
	public static SCode getCode(String category, String code) {
		Map<String, SCode> categorizedData = SCodeManager.instance().data == null ? null : SCodeManager.instance().data.get(category);
		if (categorizedData == null) {
			return null;
		} else {
			return categorizedData.get(code);
		}
	}

	/**
	 * 指定されたカテゴリのシステムコードマスタリストを取得します。<br>
	 * @param category カテゴリコード
	 * @return システムコードマスタエンティティリスト
	 */
	public static List<SCode> getList(String category, String code) {
		Map<String, SCode> categorizedData = SCodeManager.instance().data.get(category);
		if (categorizedData == null) {
			return new LinkedList<SCode>();
		} else {
			List<SCode> sCodes = new LinkedList<SCode>(categorizedData.values());
			Collections.sort(sCodes, new Comparator<SCode>() {
				@Override
				public int compare(SCode o1, SCode o2) {
					if (o1 == null && o2 == null) {
						return 0;
					} else if (o1 == null && o2 != null) {
						return -1;
					} else if (o1 != null && o2 == null) {
						return 1;
					} else {
						int i1 = o1.getOrderNo() == null ? 0 : o1.getOrderNo();
						int i2 = o2.getOrderNo() == null ? 0 : o2.getOrderNo();
						return i1 - i2;
					}
				}
			});
			return sCodes;
		}
	}
}
