package org.phosphoresce.webcore.ext.jasper.export;

import java.io.Serializable;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * 帳票を出力する際のバインドデータを保持するコンテナ機能を提供します。<br>
 * <p>
 * 当コンテナが管理するバインドデータは大きく分けて以下の2つのデータ種別となります。<br>
 * これはJasperReportsライブラリのバインドデータに沿ったデータ保持仕様です。<br>
 * 
 * <ul>
 * 
 * <li>パラメータ</li>
 * パラメータとは、JasperReportsライブラリで利用する際のパラメータと同意となります。<br>
 * 帳票定義体において明細等のイテレートされるデータレコードではなく、定義体に対して1つの値として提供されるデータとなります。<br>
 * パラメータ値は{@link Object}で保持する為、定義体及び、バインドデータの設計によってはパラメータにコレクションやマップを保持させることで、任意のユーザーによる情報保持、帳票出力が可能です。<br>
 * 但し、保持する値をコレクションやマップとすることで、内部構造が複雑になり保守性が低下する為、Beanクラスを作成して格納することが推奨されます。<br>
 * 
 * <li>データソースレコード</li>
 * データソースレコードとは、JasperReportsライブラリで利用する際のデータソース内の1つのレコードを意味します。<br>
 * データソースオブジェクト自体は当コンテナクラスが出力時に提供し、ユーザーはこのクラスが提供するレコード操作インタフェースのみを利用します。<br>
 * また、保持値はパラメータ同様に{@link Object}で保持する為、複雑な構造とならないように注意が必要です。<br>
 * 
 * </ul>
 * 
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2007/08/10	Kitagawa		新規作成
 * 2012/07/06	Kitagawa		Webcoreフレームワーク用に新たなクラスとして新規作成
 * 2013/08/13	Kitagawa		環境依存文字をフィルタしたパラメータを取得するインタフェースを追加
 *-->
 */
public class JasperContainer implements Serializable {

	/** パラメータマップオブジェクト */
	private Map<String, Object> parameter;

	/** データソースリストオブジェクト */
	private List<Map<String, Object>> datasource;

	/**
	 * コンストラクタ<br>
	 */
	public JasperContainer() {
		super();
		this.parameter = new HashMap<String, Object>();
		this.datasource = new LinkedList<Map<String, Object>>();
	}

	/**
	 * パラメータを追加します。<br>
	 * @param key パラメータキー
	 * @param value パラメータ値
	 */
	public void setParameter(String key, Object value) {
		parameter.put(key, value);
	}

	/**
	 * パラメータを追加します。<br>
	 * @param map パラメータマップ
	 */
	public void setParameters(Map<String, Object> map) {
		if (map == null) {
			return;
		}
		parameter.putAll(map);
	}

	/**
	 * パラメータを取得します。<br>
	 * @param key パラメータキー
	 * @return パラメータ値
	 */
	public Object getParameter(String key) {
		return parameter.get(key);
	}

	/**
	 * パラメータを削除します。<br>
	 * @param key パラメータキー
	 */
	public void removeParameter(String key) {
		parameter.remove(key);
	}

	/**
	 * パラメータを全て削除します。<br>
	 */
	public void clearParamater() {
		parameter.clear();
	}

	/**
	 * データソースレコードを追加します。<br>
	 * @param record データソースレコード
	 */
	public void addRecord(Map<String, Object> record) {
		datasource.add(record);
	}

	/**
	 * データソースレコードを追加します。<br>
	 * @param records データソースレコードリスト
	 */
	public void addRecords(List<Map<String, Object>> records) {
		if (records == null) {
			return;
		}
		datasource.addAll(records);
	}

	/**
	 * データソースレコードを全て削除します。<br>
	 */
	public void clearRecord() {
		datasource.clear();
	}

	/**
	 * パラメータマップオブジェクトを取得します。<br>
	 * @return パラメータマップオブジェクト
	 */
	public Map<String, Object> getParameters() {
		return parameter;
	}

	/**
	 * 環境依存文字をフィルタしたパラメータマップオブジェクトを取得します。<br>
	 * @return パラメータマップオブジェクト
	 */
	public Map<String, Object> getFilteredParameters() {
		return JasperCharacterFilter.encode(parameter);
	}

	/**
	 * データソースレコードリストオブジェクトを取得します。<br>
	 * @return データソースレコードリストオブジェクト
	 */
	public List<Map<String, Object>> getRecords() {
		return datasource;
	}

	/**
	 * 環境依存文字をフィルタしたデータソースレコードリストオブジェクトを取得します。<br>
	 * @return データソースレコードリストオブジェクト
	 */
	public List<Map<String, Object>> getFilteredRecords() {
		return JasperCharacterFilter.encode(datasource);
	}

	/**
	 * データソースレコード件数を取得します。<br>
	 * @return データソースレコード件数
	 */
	public int getRecordCount() {
		return datasource.size();
	}
}
