package org.phosphoresce.webcore.ext.jasper.export;

import java.io.Serializable;

import org.phosphoresce.webcore.ext.jasper.JasperConstants;

/**
 * 帳票出力形式のタイプセーフな列挙型定数クラスインスタンスを提供します。<br>
 * <p>
 * JasperReports拡張モジュール内で出力形式指定及び、内部処理判定において利用する列挙型定数をタイプセーフな形式で提供します。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2007/11/26	Kitagawa		新規作成
 * 2012/07/06	Kitagawa		Webcoreフレームワーク用に新たなクラスとして新規作成
 * 2013/07/19	Kitagawa		クラスフィールドをタイプ、タイム名、拡張子から、識別名、拡張子に変更
 *-->
 */
public final class JasperExportType implements Serializable, JasperConstants {

	/** 出力タイプ(PDF) */
	public static final JasperExportType PDF = new JasperExportType("PDF", ".pdf");

	/** 出力タイプ(XML) */
	public static final JasperExportType XML = new JasperExportType("XML", ".xml");

	/** 出力タイプ(CSV) */
	public static final JasperExportType CSV = new JasperExportType("CSV", ".csv");

	/** 出力タイプ(HTML) */
	public static final JasperExportType HTML = new JasperExportType("HTML", ".html");

	/** 出力タイプ(RTF) */
	public static final JasperExportType RTF = new JasperExportType("RTF", ".rtf");

	/** 出力タイプ(ODF) */
	public static final JasperExportType ODF = new JasperExportType("ODF", ".odf");

	/** 出力タイプ(XLS[JExcelAPI]) */
	public static final JasperExportType JXL = new JasperExportType("JXL", ".xls");

	/** 出力タイプ(XLS[POI]) */
	public static final JasperExportType POI = new JasperExportType("POI", ".xls");

	/** 出力タイプ(PrintService) */
	public static final JasperExportType PRINT = new JasperExportType("PRINT", null);

	/** タイプ識別ID */
	private String id;

	/** 拡張子 */
	private String extension;

	/**
	 * コンストラクタ<br>
	 * @param id タイプ識別ID
	 * @param extension 出力ファイル拡張子
	 */
	private JasperExportType(String id, String extension) {
		super();
		this.id = id;
		this.extension = extension;
	}

	/**
	 * 指定されたファイル名を出力タイプに合致するファイル名として提供します。<br>
	 * @param filename 対象ファイル名
	 * @return 出力タイプに合致するファイル名
	 */
	public String validateFilename(String filename) {
		if (filename.toLowerCase().endsWith(extension.toLowerCase())) {
			return filename;
		} else {
			return filename + extension.toLowerCase();
		}
	}

	/**
	 * オブジェクトハッシュコードを算出します。<br>
	 * @return オブジェクトハッシュコード
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((extension == null) ? 0 : extension.hashCode());
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		return result;
	}

	/**
	 * オブジェクトの比較を行います。<br>
	 * @param object 比較対象オブジェクト
	 * @return 同値のオブジェクトの場合にtrueを返却
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(Object object) {
		if (this == object) {
			return true;
		}
		if (object == null) {
			return false;
		}
		if (getClass() != object.getClass()) {
			return false;
		}
		JasperExportType other = (JasperExportType) object;
		if (extension == null) {
			if (other.extension != null) {
				return false;
			}
		} else if (!extension.equals(other.extension)) {
			return false;
		}
		if (id == null) {
			if (other.id != null) {
				return false;
			}
		} else if (!id.equals(other.id)) {
			return false;
		}
		return true;
	}

	/**
	 * クラスを文字列表現で取得します。<br>
	 * @return 文字列表現
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return id;
	}
}
