package org.phosphoresce.webcore.ext.jasper.export;

import java.util.LinkedList;
import java.util.List;

import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRExporter;
import net.sf.jasperreports.engine.JRExporterParameter;
import net.sf.jasperreports.engine.JRPrintPage;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import org.phosphoresce.webcore.ext.jasper.exception.JasperCriticalException;
import org.phosphoresce.webcore.ext.jasper.exception.JasperNoReportException;

/**
 * 複数定義体から単一の帳票として出力する為の出力インタフェースを提供します。<br>
 * <p>
 * JasperReportsを利用した帳票出力において、複数の定義体から単一の帳票として出力する際の煩雑なコーディングを回避する為に設けられました。<br>
 * このクラスは{@link JasperWriter}クラスを継承したクラスとなっている為、通常の単一帳票出力操作時と同様の出力操作インタフェースが保証されます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2007/08/10	Kitagawa		新規作成
 * 2012/07/06	Kitagawa		Webcoreフレームワーク用に新たなクラスとして新規作成
 * 2013/08/13	Kitagawa		帳票へのデータバインド時に環境依存文字をフィルタしてセットするように修正
 *-->
 */
public class JasperMultipageWriter extends JasperWriter {

	/** 帳票定義パス */
	private List<String> jrxmlPaths;

	/** 帳票データコンテナオブジェクト */
	private List<JasperContainer> containers;

	/**
	 * コンストラクタ<br>
	 */
	public JasperMultipageWriter() {
		super();
		this.jrxmlPaths = new LinkedList<String>();
		this.containers = new LinkedList<JasperContainer>();
	}

	/**
	 * 指定されたパスの定義体及び出力帳票情報を追加します。<br>
	 * @param jrxmlPath 帳票定義パス
	 * @param container 帳票データコンテナオブジェクト
	 */
	public void add(String jrxmlPath, JasperContainer container) {
		if (jrxmlPath == null) {
			throw new NullPointerException("jrxmlPath");
		}
		if (container == null) {
			throw new NullPointerException("container");
		}
		this.jrxmlPaths.add(jrxmlPath);
		this.containers.add(container);
	}

	/**
	 * コンパイル済み帳票定義オブジェクトを生成します。<br>
	 * @return 帳票定義オブジェクト
	 * @throws JRException 正常に帳票定義オブジェクトを生成できなかった場合にスローされます
	 */
	private List<JasperReport> createJasperReport() throws JRException {
		List<JasperReport> list = new LinkedList<JasperReport>();
		for (String jrxmlPath : jrxmlPaths) {
			if (jrxmlPath.endsWith(".jasper")) {
				list.add(null);
			} else {
				list.add(JasperCompileManager.compileReport(jrxmlPath));
			}
		}
		return list;
	}

	/**
	 * データソースオブジェクトを生成します。<br>
	 * @return データソースオブジェクト
	 */
	private List<JRDataSource> createJRDataSource() {
		List<JRDataSource> list = new LinkedList<JRDataSource>();
		for (JasperContainer container : containers) {
			list.add(new JRBeanCollectionDataSource(container.getFilteredRecords()));
		}
		return list;
	}

	/**
	 * 出力オブジェクトを生成します。<br>
	 * @return 出力オブジェクト
	 * @throws JRException 正常に出力オブジェクトを生成できなかった場	合にスローされます
	 */
	private List<JasperPrint> createJasperPrint() throws JRException {
		List<JasperPrint> list = new LinkedList<JasperPrint>();
		List<JasperReport> reports = createJasperReport();
		List<JRDataSource> sources = createJRDataSource();
		int count = 0;
		for (JasperReport report : reports) {
			if (report == null) {
				list.add(JasperFillManager.fillReport(jrxmlPaths.get(count), containers.get(count).getFilteredParameters(), sources.get(count)));
			} else {
				list.add(JasperFillManager.fillReport(report, containers.get(count).getFilteredParameters(), sources.get(count)));
			}
			count++;
		}
		return list;
	}

	/**
	 * 自身が保持しているパラメータオブジェクトを指定されたJRExporterに対して設定します。<br>
	 * @param exporter JRExporterオブジェクト
	 * @see org.phosphoresce.webcore.ext.jasper.export.JasperWriter#fillParameter(net.sf.jasperreports.engine.JRExporter)
	 */
	@Override
	protected void fillParameter(JRExporter exporter) {
		try {
			List<JasperPrint> prints = createJasperPrint();
			if (prints.size() <= 0) {
				return;
			} else if (prints.size() == 1) {
				exporter.setParameter(JRExporterParameter.JASPER_PRINT, prints.get(0));
			} else {
				JasperPrint print = prints.get(0);
				for (int i = 1; i <= prints.size() - 1; i++) {
					for (JRPrintPage page : prints.get(i).getPages()) {
						print.addPage(page);
					}
				}
				exporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
			}
		} catch (Throwable e) {
			throw new JasperCriticalException("FJPR00009", e);
		}
	}

	/**
	 * 帳票のエクスポート処理を行います。<br>
	 * 当メソッドは上位の{@link JasperWriter#exportProcessWrapper(JRExporter)}がオーバーライドされ、出力対象の帳票が1件も追加されていない場合に
	 * アプリケーション例外として{@link JasperNoReportException}がスローされます。<br>
	 * 但し、これはデータソースのレコードが存在するかどうかの判定ではなく、結合帳票出力時に定義体が1つも追加されていない場合に発生するのみであることに注意して下さい。<br>
	 * @param exporter エクスポーターオブジェクト
	 * @throws 処理時に予期せぬ例外が発生した場合にスローされます
	 * @see org.phosphoresce.webcore.ext.jasper.export.JasperWriter#exportProcessWrapper(net.sf.jasperreports.engine.JRExporter)
	 */
	@Override
	protected void exportProcessWrapper(JRExporter exporter) throws Throwable {
		if (containers.size() <= 0) {
			throw new JasperNoReportException();
		} else {
			super.exportProcessWrapper(exporter);
		}
	}
}
