package org.phosphoresce.webcore.ext.jasper.export;

import net.sf.jasperreports.engine.JRDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRExporter;
import net.sf.jasperreports.engine.JRExporterParameter;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;

import org.phosphoresce.webcore.ext.jasper.exception.JasperCriticalException;

/**
 * 1つの定義体から帳票出力する為の出力インタフェースを提供します。<br>
 * <p>
 * JasperReportsを利用した帳票出力において、定義体に対して1つの帳票を出力する際の単純な出力インタフェースが提供されます。<br>
 * このクラスは{@link JasperWriter}クラスを継承したクラスとなっている為、他の帳票出力操作時と同様の出力操作インタフェースが保証されます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2007/08/10	Kitagawa		新規作成
 * 2012/07/06	Kitagawa		Webcoreフレームワーク用に新たなクラスとして新規作成
 * 2013/08/13	Kitagawa		帳票へのデータバインド時に環境依存文字をフィルタしてセットするように修正
 *-->
 */
public class JasperSingleWriter extends JasperWriter {

	/** 帳票定義パス */
	private String jrxmlPath;

	/** 帳票データコンテナオブジェクト */
	private JasperContainer container;

	/**
	 * コンストラクタ<br>
	 */
	protected JasperSingleWriter() {
		super();
	}

	/**
	 * コンストラクタ<br>
	 * @param jrxmlPath 帳票定義パス
	 * @param container 帳票データコンテナオブジェクト
	 */
	public JasperSingleWriter(String jrxmlPath, JasperContainer container) {
		super();
		this.jrxmlPath = jrxmlPath;
		this.container = container;
	}

	/**
	 * コンパイル済み帳票定義オブジェクトを生成します。<br>
	 * @return 帳票定義オブジェクト
	 * @throws JRException 正常に帳票定義オブジェクトを生成できなかった場合にスローされます
	 */
	private JasperReport createJasperReport() throws JRException {
		if (jrxmlPath.endsWith(".jasper")) {
			return null;
		}
		return JasperCompileManager.compileReport(jrxmlPath);
	}

	/**
	 * データソースオブジェクトを生成します。<br>
	 * @return データソースオブジェクト
	 */
	private JRDataSource createJRDataSource() {
		return new JRBeanCollectionDataSource(container.getFilteredRecords());
	}

	/**
	 * 出力オブジェクトを生成します。<br>
	 * @return 出力オブジェクト
	 * @throws JRException 正常に出力オブジェクトを生成できなかった場	合にスローされます
	 */
	private JasperPrint createJasperPrint() throws JRException {
		JasperReport report = createJasperReport();
		if (report == null) {
			return JasperFillManager.fillReport(jrxmlPath, container.getFilteredParameters(), createJRDataSource());
		} else {
			return JasperFillManager.fillReport(report, container.getFilteredParameters(), createJRDataSource());
		}
	}

	/**
	 * 自身が保持しているパラメータオブジェクトを指定されたJRExporterに対して設定します。<br>
	 * @param exporter JRExporterオブジェクト
	 * @see org.phosphoresce.webcore.ext.jasper.export.JasperWriter#fillParameter(net.sf.jasperreports.engine.JRExporter)
	 */
	@Override
	protected void fillParameter(JRExporter exporter) {
		try {
			JasperPrint print = createJasperPrint();
			exporter.setParameter(JRExporterParameter.JASPER_PRINT, print);
		} catch (Throwable e) {
			throw new JasperCriticalException("FJPR00009", e);
		}
	}
}
