package org.phosphoresce.webcore.ext.struts.action;

import org.phosphoresce.webcore.core.GlobalConstants;
import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.phosphoresce.webcore.core.transaction.TransactionUser;
import org.phosphoresce.webcore.ext.struts.StrutsConstants;
import org.phosphoresce.webcore.ext.struts.exception.StrutsModuleApplicationException;
import org.phosphoresce.webcore.ext.struts.exception.StrutsModuleCriticalException;
import org.slf4j.LoggerFactory;

/**
 * 画面遷移時のディスパッチ処理を行うアクションクラスで実行する処理を行う抽象的なフックメソッドを提供します。<br>
 * <p>
 * アクションクラスにおける、ディスパッチ処理以外の詳細な処理プロセスを切り分けて実装するために設けられたクラスです。<br>
 * これは内部的な処理を実現するために利用されるクラスで、実際のアプリケーション開発を行うユーザーは意識する必要はありません。<br>
 * </p>
 * </p>
 * このクラスではアクション処理時の実処理の呼び出し部及び、例外ハンドル処理等のフックメソッドがabstractで提供され、これを継承したクラスにて実装します。<br>
 * 将来的にディスパッチ処理時の実処理の変更や拡張を柔軟に行うために、インタフェースのみが決定されて、提供されます。<br>
 * </p>
 * 
 * @param <L> アプリケーション実行ユーザーのトランザクション処理ユーザークラス
 * @param <F> アクションに対応するフォームクラス
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2013/07/12	Kitagawa		新規作成({@link AbstractAction}から内部処理部を分割)
 *-->
 */
abstract class AbstractActionProcess<L extends TransactionUser, F extends AbstractForm<L>> implements GlobalConstants, StrutsConstants {

	/** ロガーオブジェクト */
	protected CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(getClass()));

	/**
	 * アクション処理を実行します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @throws Throwable アクション処理中に例外が発生した場合にスローされます
	 */
	public abstract void executeProcess(ActionContext<L, F> actionContext) throws Throwable;

	/**
	 * アクション終了処理を実行します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @throws Throwable アクション処理処理中に例外が発生した場合にスローされます
	 */
	public abstract void executeFinally(ActionContext<L, F> actionContext) throws Throwable;

	/**
	 * アドオンモジュールアプリケーション例外発生時のハンドル処理を実行します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @param e 例外オブジェクト
	 * @throws Throwable 例外ハンドル処理中に別の例外が発生した場合にスローされます(この場合はサーブレットエラーとして扱われます)
	 */
	public abstract void handleStrutsModuleApplicationException(ActionContext<L, F> actionContext, StrutsModuleApplicationException e) throws Throwable;

	/**
	 * アドオンモジュール致命的例外発生時のハンドル処理を実行します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @param e 例外オブジェクト
	 * @throws Throwable 例外ハンドル処理中に別の例外が発生した場合にスローされます(この場合はサーブレットエラーとして扱われます)
	 */
	public abstract void handleStrutsCriticalModuleException(ActionContext<L, F> actionContext, StrutsModuleCriticalException e) throws Throwable;

	/**
	 * アドオンモジュール例外以外の例外発生時のハンドル処理を実行します。<br>
	 * @param actionContext アクションコンテキストオブジェクト
	 * @param e 例外オブジェクト
	 * @throws Throwable 例外ハンドル処理中に別の例外が発生した場合にスローされます(この場合はサーブレットエラーとして扱われます)
	 */
	public abstract void handleThrowable(ActionContext<L, F> actionContext, Throwable e) throws Throwable;
}
