package org.phosphoresce.webcore.ext.struts.action;

import java.lang.reflect.Constructor;
import java.util.Date;

import org.phosphoresce.webcore.core.logger.CodeConvertLogger;
import org.phosphoresce.webcore.core.transaction.TransactionContext;
import org.phosphoresce.webcore.core.transaction.TransactionManager;
import org.phosphoresce.webcore.core.transaction.TransactionService;
import org.phosphoresce.webcore.core.transaction.TransactionUser;
import org.phosphoresce.webcore.ext.struts.StrutsConstants;
import org.phosphoresce.webcore.ext.struts.annotation.StrutsService;
import org.phosphoresce.webcore.ext.struts.exception.StrutsProcessInternalException;
import org.phosphoresce.webcore.ext.struts.exception.StrutsSessionTimeoutException;
import org.phosphoresce.webcore.ext.struts.util.StrutsAnnotationUtil;
import org.slf4j.LoggerFactory;

/**
 * アクションにおける業務処理を実装するための共通のインタフェースを提供します。<br>
 * <p>
 * トランザクションを必要とする業務処理を実装するクラスはこのクラスを継承したロジッククラスに限定されます。<br>
 * 当クラスはトランザクション処理を可能とするクラスである反面、アクションレイヤスコープであるディスパッチ処理を行うことが制限されています。<br>
 * 制限されるアクションレイヤスコープで実行する処理については{@link ServletContainer}が提供するインタフェースのうち、レスポンス処理等も同様で、
 * {@link ServletContainer}に対してこれらの操作を呼び出した場合、意図的に例外がスローされることに注意してください。<br>
 * </p>
 * <p>
 * また、ディスパッチイベント単位でトランザクションが単一構成であることを前提とし、リダイレクトするサーブレット間でのトランザクション共有は対応しません。<br>
 * これはトランザクション管理を曖昧にすることなく、サーブレット単位で完結させるための思想であり、リダイレクトするサーブレット間でのトランザクション共有が必要な設計を不正と位置付けます。<br>
 * 但し、ロジッククラス処理のチェーンは思想上も許可しているため、業務処理設計でトランザクションを共有する必要がある場合は、ロジッククラスレイヤ間で業務処理を関連付ける設計とすることで対応します。<br>
 * </p>
 * 
 * @param <L> アプリケーション実行ユーザーのトランザクション処理ユーザークラス
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2010/07/19	Kitagawa		新規作成
 * 2012/07/05	Kitagawa		全体的に再構築
 * 2013/07/24	Kitagawa		コンストラクタによるインスタンス生成を不可とする為に{@link ServletContainer}の設定は初期化処理メソッドで行うように修正
 *-->
 */
public abstract class AbstractLogic<L extends TransactionUser> implements StrutsConstants {

	/** ロガーオブジェクト */
	protected CodeConvertLogger log = new CodeConvertLogger(LoggerFactory.getLogger(getClass()));

	/** サーブレットコンテナオブジェクト */
	private ServletContainer<L> servlet;

	/** 初期化済みフラグ */
	private boolean initialized;

	/**
	 * コンストラクタ<br>
	 */
	public AbstractLogic() {
		super();
		this.servlet = null;
		this.initialized = false;
	}

	/**
	 * ロジッククラスを初期化します。<br>
	 * @param servlet サーブレットコンテナオブジェクト
	 */
	final void initialize(ServletContainer<L> servlet) {
		this.servlet = servlet;
		initialized = true;

		// 追加初期化処理実行
		init(servlet);
	}

	/**
	 * ロジッククラスの初期化処理を実行します。<br>
	 * @param servlet サーブレットコンテナオブジェクト
	 */
	protected abstract void init(ServletContainer<L> servlet);

	/**
	 * ロジッククラスが初期化済みであるか判定します。<br>
	 */
	private void checkInitialized() {
		if (!initialized) {
			throw new StrutsProcessInternalException("FSTR07003");
		}
	}

	/**
	 * 現在時刻を取得します。<br>
	 * @return 現在時刻
	 */
	protected final Date getProcessDate() {
		checkInitialized();

		//return new Date();
		return servlet.getProcessDate();
	}

	/**
	 * サーブレットコンテナオブジェクトを取得します。<br>
	 * @return サーブレットコンテナオブジェクト
	 */
	protected final ServletContainer<L> getServlet() {
		checkInitialized();

		return new ServletContainerLogicWrapper<L>(servlet);
	}

	/**
	 * ウィンドウセッションオブジェクトを取得します。<br>
	 * @return ウィンドウセッションオブジェクト
	 */
	protected final WindowSessionContainer<L> getSession() {
		checkInitialized();

		WindowSessionContainer<L> session = servlet.getSession().getWindowSession(servlet.getWindowId());
		if (session == null) {
			throw new StrutsSessionTimeoutException();
		}
		return servlet.getSession().getWindowSession(servlet.getWindowId());
	}

	/**
	 * トランザクションコンテキストクラスインスタンスを生成します。<br>
	 * @param <C> トランザクションコンテキストクラス
	 * @param managerClass トランザクションマネージャクラス
	 * @param userClass トランザクションユーザークラス
	 * @param contextClass トランザクションコンテキストクラス
	 * @param lookup トランザクションマネージャルックアップ名
	 * @return トランザクションコンテキストクラスインスタンス
	 */
	private final <C extends TransactionContext<?, ?>> C createTransactionContext(Class<? extends TransactionManager> managerClass, Class<? extends TransactionUser> userClass, Class<C> contextClass, String lookup) {
		checkInitialized();

		try {
			TransactionManager transaction = null;
			if (lookup == null) {
				transaction = servlet.getTransaction().get(DEFAULT_TRANSACTION_LOOKUP_NAME);
			} else {
				transaction = servlet.getTransaction().get(lookup);
			}
			Date processDate = getServlet().getProcessDate();
			L processUser = getSession().getLoginUser() == null ? null : getSession().getLoginUser().getLoginInfo();
			Constructor<C> constructor = contextClass.getConstructor(managerClass, userClass, Date.class);
			return constructor.newInstance(transaction, processUser, processDate);
		} catch (Throwable e) {
			throw new StrutsProcessInternalException("FSTR00018", new Object[] { (contextClass == null ? "null" : contextClass.getSimpleName()) }, e);
		}
	}

	/**
	 * 指定されたルックアップ名のトランザクションを利用するサービスクラスを生成します。<br>
	 * @param clazz サービスクラス
	 * @param lookup トランザクションルックアップ名
	 * @return サービスクラスインスタンス
	 */
	protected final <S extends TransactionService<?, L, ?>> S createService(Class<S> clazz, String lookup) {
		checkInitialized();

		try {
			StrutsService strutsService = StrutsAnnotationUtil.getStrutsServiceAnnotation(clazz);
			if (strutsService == null) {
				throw new StrutsProcessInternalException("FSTR12001", new Object[] { (clazz == null ? "null" : clazz.getSimpleName()) });
			}
			Constructor<S> constructor = clazz.getConstructor(strutsService.transactionContext());
			S s = constructor.newInstance(createTransactionContext(strutsService.transactionManager(), strutsService.transactionUser(), strutsService.transactionContext(), lookup));
			servlet.addService(s);
			return s;
		} catch (Throwable e) {
			throw new StrutsProcessInternalException("FSTR12002", new Object[] { (clazz == null ? "null" : clazz.getSimpleName()) }, e);
		}
	}

	/**
	 * ディフォルトのトランザクションを利用するサービスクラスを生成します。<br>
	 * @param clazz サービスクラス
	 * @return サービスクラスインスタンス
	 */
	protected final <S extends TransactionService<?, L, ?>> S createService(Class<S> clazz) {
		checkInitialized();

		return createService(clazz, null);
	}

	/**
	 * ロジッククラスを生成します。<br>
	 * 自身のロジッククラスから別のロジッククラスを利用する際に利用しますが、クラス間の結合度が高くなるため、
	 * 相互利用を行う際のクラス設計が煩雑にならないように注意してください。<br>
	 * @param clazz ロジッククラス
	 * @return ロジッククラスインスタンス
	 */
	protected final <G extends AbstractLogic<L>> G createLogic(Class<G> clazz) {
		checkInitialized();

		try {
			if (StrutsAnnotationUtil.getStrutsLogicAnnotation(clazz) == null) {
				throw new StrutsProcessInternalException("FSTR07001", new Object[] { (clazz == null ? "null" : clazz.getSimpleName()) });
			}
			Constructor<G> constructor = clazz.getConstructor();
			G g = constructor.newInstance();
			g.initialize(servlet);
			servlet.addLogic(g);
			return g;
		} catch (Throwable e) {
			throw new StrutsProcessInternalException("FSTR07002", new Object[] { (clazz == null ? "null" : clazz.getSimpleName()) }, e);
		}
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessage(String code, Object... binds) {
		checkInitialized();

		servlet.getMessage().addMessage(code, binds);
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessageOnHead(String code, Object... binds) {
		checkInitialized();

		servlet.getMessage().addMessageOnHead(code, binds);
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param code メッセージコード
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessage(String code) {
		checkInitialized();

		servlet.getMessage().addMessage(code);
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param code メッセージコード
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessageOnHead(String code) {
		checkInitialized();

		servlet.getMessage().addMessageOnHead(code);
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessage(Integer row, String code, Object... binds) {
		checkInitialized();

		servlet.getMessage().addMessage(row, code, binds);
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessageOnHead(Integer row, String code, Object... binds) {
		checkInitialized();

		servlet.getMessage().addMessageOnHead(row, code, binds);
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessage(Integer row, String code) {
		checkInitialized();

		servlet.getMessage().addMessage(row, code);
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 * @see org.phosphoresce.webcore.ext.struts.action.MessageContainer
	 */
	public final void addMessageOnHead(Integer row, String code) {
		checkInitialized();

		servlet.getMessage().addMessageOnHead(row, code);
	}
}
