package org.phosphoresce.webcore.ext.struts.action;

import java.io.Serializable;
import java.util.LinkedList;
import java.util.List;

import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;
import org.phosphoresce.webcore.core.GlobalConfigName;
import org.phosphoresce.webcore.core.config.Config;
import org.phosphoresce.webcore.core.config.StringResource;
import org.phosphoresce.webcore.core.config.StringResourceLevel;

/**
 * アプリケーション上で発生する各種メッセージを管理するコンテナインタフェースを提供します。<br>
 * <p>
 * Strutsのアクションメッセージによる個別のメッセージ設定はアドオンモジュールでは行いません。<br>
 * 代わりにここで提供されるメッセージコンテナクラスインスタンスがアクション実行中の全てのレイヤで利用され、一元的に管理されます。<br>
 * </p>
 * <p>
 * 尚、メッセージレベルは{@link org.phosphoresce.webcore.core.logger.CodeConvertLogger}の思想を継承している為、メッセージレベル毎のメッセージ操作インタフェースは提供されません。<br>
 * メッセージレベルは外部定義されたメッセージコードに依存して自動的に決定されます。<br>
 * </p>
 * 
 * @author Kitagawa<br>
 * 
 *<!--
 * 更新日		更新者			更新内容
 * 2009/12/18	Kitagawa		新規作成
 * 2012/07/06	Kitagawa		全体的に再構築
 * 2013/05/28	Kitagawa		先頭へのメッセージ追加インタフェースを追加
 * 2013/06/18	Kitagawa		メッセージレベル毎のインタフェースを削除(統合してコードによる分岐)
 * 2013/07/24	Kitagawa		{@link org.phosphoresce.webcore.core.logger.CodeConvertLogger}に合わせて非表示ログレベルに対応
 *-->
 */
public final class MessageContainer implements Serializable {

	/** エラーメッセージ */
	private List<MessageSet> errorMessages = new LinkedList<MessageSet>();

	/** 情報メッセージ */
	private List<MessageSet> informationMessages = new LinkedList<MessageSet>();

	/** 警告メッセージ */
	private List<MessageSet> warningMessages = new LinkedList<MessageSet>();

	/** 連絡メッセージ */
	private List<MessageSet> noticeMessages = new LinkedList<MessageSet>();

	/** デバッグメッセージ */
	private List<MessageSet> debugMessages = new LinkedList<MessageSet>();

	/** トレースメッセージ */
	private List<MessageSet> traceMessages = new LinkedList<MessageSet>();

	/** 非表示メッセージ */
	private List<MessageSet> hiddenMessages = new LinkedList<MessageSet>();

	/** その他メッセージ */
	private List<MessageSet> otherMessages = new LinkedList<MessageSet>();

	/**
	 * コンストラクタ<br>
	 */
	MessageContainer() {
		super();
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param set メッセージセットオブジェクト
	 */
	public final void addMessage(MessageSet set) {
		if (set == null) {
			return;
		}
		StringResourceLevel level = StringResourceLevel.getDefinitionLevel(set.getCode());
		if (StringResourceLevel.INFO.equals(level)) {
			if (!informationMessages.contains(set)) {
				informationMessages.add(set);
			}
		} else if (StringResourceLevel.ERROR.equals(level)) {
			if (!errorMessages.contains(set)) {
				errorMessages.add(set);
			}
		} else if (StringResourceLevel.WARN.equals(level)) {
			if (!warningMessages.contains(set)) {
				warningMessages.add(set);
			}
		} else if (StringResourceLevel.NOTICE.equals(level)) {
			if (!noticeMessages.contains(set)) {
				noticeMessages.add(set);
			}
		} else if (StringResourceLevel.DEBUG.equals(level)) {
			if (!debugMessages.contains(set)) {
				debugMessages.add(set);
			}
		} else if (StringResourceLevel.TRACE.equals(level)) {
			if (!traceMessages.contains(set)) {
				traceMessages.add(set);
			}
		} else if (StringResourceLevel.HIDDEN.equals(level)) {
			if (!hiddenMessages.contains(set)) {
				hiddenMessages.add(set);
			}
		} else {
			if (!otherMessages.contains(set)) {
				otherMessages.add(set);
			}
		}
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param set メッセージセットオブジェクト
	 */
	public final void addMessageOnHead(MessageSet set) {
		if (set == null) {
			return;
		}
		StringResourceLevel level = StringResourceLevel.getDefinitionLevel(set.getCode());
		if (StringResourceLevel.INFO.equals(level)) {
			if (!informationMessages.contains(set)) {
				informationMessages.add(set);
			}
		} else if (StringResourceLevel.ERROR.equals(level)) {
			if (!errorMessages.contains(set)) {
				errorMessages.add(set);
			}
		} else if (StringResourceLevel.WARN.equals(level)) {
			if (!warningMessages.contains(set)) {
				warningMessages.add(set);
			}
		} else if (StringResourceLevel.NOTICE.equals(level)) {
			if (!noticeMessages.contains(set)) {
				noticeMessages.add(set);
			}
		} else if (StringResourceLevel.DEBUG.equals(level)) {
			if (!debugMessages.contains(set)) {
				debugMessages.add(set);
			}
		} else if (StringResourceLevel.TRACE.equals(level)) {
			if (!traceMessages.contains(set)) {
				traceMessages.add(set);
			}
		} else if (StringResourceLevel.HIDDEN.equals(level)) {
			if (!hiddenMessages.contains(set)) {
				hiddenMessages.add(set);
			}
		} else {
			if (!otherMessages.contains(set)) {
				otherMessages.add(set);
			}
		}
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 */
	public final void addMessage(Integer row, String code, Object... binds) {
		addMessage(new MessageSet(row, code, binds));
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 */
	public final void addMessageOnHead(Integer row, String code, Object... binds) {
		addMessageOnHead(new MessageSet(row, code, binds));
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 */
	public final void addMessage(Integer row, String code) {
		addMessage(new MessageSet(row, code, null));
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param row 明細行数
	 * @param code メッセージコード
	 */
	public final void addMessageOnHead(Integer row, String code) {
		addMessageOnHead(new MessageSet(row, code, null));
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 */
	public final void addMessage(String code, Object... binds) {
		addMessage(new MessageSet(null, code, binds));
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param code メッセージコード
	 */
	public final void addMessageOnHead(String code) {
		addMessageOnHead(new MessageSet(null, code, null));
	}

	/**
	 * アプリケーションメッセージを追加します。<br>
	 * @param code メッセージコード
	 */
	public final void addMessage(String code) {
		addMessage(new MessageSet(null, code, null));
	}

	/**
	 * アプリケーションメッセージを先頭に追加します。<br>
	 * @param code メッセージコード
	 * @param binds バインドオブジェクト
	 */
	public final void addMessageOnHead(String code, Object... binds) {
		addMessageOnHead(new MessageSet(null, code, binds));
	}

	/**
	 * 指定されたメッセージセットリストから名前空間通りのアクションメッセージを取得します。<br>
	 * @param targetMessages メッセージリスト
	 * @param namespace 名前空間文字列
	 * @return アクションメッセージオブジェクト
	 */
	private final ActionMessages getActionMessages(List<MessageSet> targetMessages, String namespace) {
		ActionMessages messages = new ActionMessages();
		if (targetMessages != null) {
			for (MessageSet set : targetMessages) {
				String string = "";
				if (set.getRowNumber() != null) {
					string = StringResource.getString("FCRV00001", set.getRowNumber(), StringResource.getString(set.getCode(), set.getBinds()));
				} else {
					string = StringResource.getString(set.getCode(), set.getBinds());
				}
				ActionMessage message = null;
				if (Config.getBoolean(GlobalConfigName.COMMON_MESSAGE_SHOWCODE)) {
					message = new ActionMessage(namespace + ".general", "[" + set.getCode() + "] " + string);
				} else {
					message = new ActionMessage(namespace + ".general", string);
				}
				messages.add(namespace, message);
			}
		}
		return messages;
	}

	/**
	 * すべてのレベルのメッセージを取得します。<br>
	 * @return アクションメッセージ
	 */
	final ActionMessages getAllActionMessages() {
		final ActionMessages messages = new ActionMessages();
		messages.add(getInformationActionMessages());
		messages.add(getErrorActionMessages());
		messages.add(getWarningActionMessages());
		messages.add(getNoticeActionMessages());
		messages.add(getDebugActionMessages());
		messages.add(getTraceActionMessages());
		messages.add(getHiddenActionMessages());
		messages.add(getOtherActionMessages());
		return messages;
	}

	/**
	 * 情報アクションメッセージを取得します。<br>
	 * @return 情報アクションメッセージ
	 */
	final ActionMessages getInformationActionMessages() {
		return getActionMessages(informationMessages, "informations");
	}

	/**
	 * エラーアクションメッセージを取得します。<br>
	 * @return エラーアクションメッセージ
	 */
	final ActionMessages getErrorActionMessages() {
		return getActionMessages(errorMessages, "errors");
	}

	/**
	 * 警告アクションメッセージを取得します。<br>
	 * @return 警告アクションメッセージ
	 */
	final ActionMessages getWarningActionMessages() {
		return getActionMessages(warningMessages, "warnings");
	}

	/**
	 * 連絡アクションメッセージを取得します。<br>
	 * @return 警告アクションメッセージ
	 */
	final ActionMessages getNoticeActionMessages() {
		return getActionMessages(noticeMessages, "notices");
	}

	/**
	 * デバッグアクションメッセージを取得します。<br>
	 * @return デバッグアクションメッセージ
	 */
	final ActionMessages getDebugActionMessages() {
		return getActionMessages(debugMessages, "debugs");
	}

	/**
	 * トレースアクションメッセージを取得します。<br>
	 * @return トレースアクションメッセージ
	 */
	final ActionMessages getTraceActionMessages() {
		return getActionMessages(traceMessages, "traces");
	}

	/**
	 * 非表示アクションメッセージを取得します。<br>
	 * @return 非表示アクションメッセージ
	 */
	final ActionMessages getHiddenActionMessages() {
		return getActionMessages(hiddenMessages, "hiddens");
	}

	/**
	 * その他アクションメッセージを取得します。<br>
	 * @return その他アクションメッセージ
	 */
	final ActionMessages getOtherActionMessages() {
		return getActionMessages(otherMessages, "others");
	}
}
